/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        Copyright (c) Non, Inc. 1998 -- All Rights Reserved

PROJECT:        C Compiler
MODULE:         GnuCEmitter_print
FILE:           GnuCEmitter_print.g

AUTHOR:         Monty Zukowski (jamz@cdsnet.net) April 28, 1998

DESCRIPTION:

                This tree grammar is for a Gnu C AST.
                It turns the tree back into source code.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

{
import java.io.*;
import java.util.*;

import antlr.CommonAST;
import antlr.DumpASTVisitor;
}

class GnuCEmitter_print extends GnuCTreeParser;

options
        {
        importVocab = GNUC;
        buildAST = false;
        ASTLabelType = "TNode";

        // Copied following options from java grammar.
        codeGenMakeSwitchThreshold = 2;
        codeGenBitsetTestThreshold = 3;
        }

{

boolean DeBug = false;
int counter = 0;
String MyTokenSep = "\n";
String Open = "<<OPEN>>";
String Close = "<<CLOSE>>";
String NewlineToken = "<<NEWLINE>>";
String TabToken = "<<TAB>>";
String PreprocSeparatorToken = "<<PREPROC>>";
int NONE = 0;
int SOME = 1;
int ALL = 2;
int TOKENONLY = 1; 
int FULL = 2;
int DoPreproc = 0;

int tabs = 0;
PrintStream currentOutput = System.out;
int lineNum = 1;
String currentSource = "";
LineObject trueSourceFile;
final int lineDirectiveThreshold = Integer.MAX_VALUE;
PreprocessorInfoChannel preprocessorInfoChannel = null;
Stack sourceFiles = new Stack();

GnuCEmitter_print( PreprocessorInfoChannel preprocChannel )
{
        preprocessorInfoChannel = preprocChannel;
}

void initializePrinting()
{
    Vector preprocs = preprocessorInfoChannel.extractLinesPrecedingTokenNumber( new Integer(1) );
    printPreprocs(preprocs);
}

void finalizePrinting() {
    // flush any leftover preprocessing instructions to the stream

    printPreprocs(
        preprocessorInfoChannel.extractLinesPrecedingTokenNumber(
                new Integer( preprocessorInfoChannel.getMaxTokenNumber() + 1 ) ));
    // print a newline so file ends with a new line
    currentOutput.println();
}

void printPreprocs( Vector preprocs )
{
    // if there was a preprocessingDirective previous to this token then
    // print a newline and the directive, line numbers handled later
    if ( preprocs.size() > 0 ) {
        currentOutput.print(NewlineToken + MyTokenSep);
        lineNum++;
        Enumeration e = preprocs.elements();
        while (e.hasMoreElements())
        {
            // In the original "GnuCEmitter_print.g", line number directives were
            // treated specially. Not here.
            Object o = e.nextElement();
            if (DoPreproc == FULL)
                currentOutput.print( Open + MyTokenSep +
                      "preproc_directive" + MyTokenSep +
                      o + MyTokenSep +
                      Close + MyTokenSep +
                      NewlineToken + MyTokenSep);
            else if (DoPreproc == TOKENONLY)
                currentOutput.print( o + MyTokenSep +
                                     NewlineToken + MyTokenSep);
            lineNum++;
        }
    }
}

// Called for C tokens. Inserts "<<NEWLINE>>" and "<<TAB>>" where they belong.
void print( TNode t ) {
    int tLineNum = t.getLocalLineNum();
    if ( tLineNum == 0 ) tLineNum = lineNum;

    Vector preprocs = preprocessorInfoChannel.extractLinesPrecedingTokenNumber((Integer)t.getAttribute("tokenNumber"));
    printPreprocs(preprocs);

    if ( (lineNum != tLineNum) ) {
        // we know we'll be newlines or a line directive or it probably
        // is just the case that this token is on the next line
        // either way start a new line and indent it
        currentOutput.print(NewlineToken + MyTokenSep);
        lineNum++;
        printTabs();
    }

    if ( lineNum == tLineNum ){
        // do nothing special, we're at the right place
    }
    else {
        int diff = tLineNum - lineNum;
        if ( lineNum < tLineNum ) {
            // print out the blank lines to bring us up to right line number
            for ( ; lineNum < tLineNum ; lineNum++ ) {
                currentOutput.print(NewlineToken + MyTokenSep);
            }
            printTabs();
        }
        else { // just reset lineNum
            lineNum = tLineNum;
        }
    }
    // This is called when printing C tokens.
    currentOutput.print( t.getText() + MyTokenSep );
}

/** It is not ok to print newlines from the String passed in as
it will screw up the line number handling **/
// Affects printing node names only.
void print( String s ) {
    currentOutput.print( s + MyTokenSep);
}

void printTabs() {
    for ( int i = 0; i< tabs; i++ ) {
        currentOutput.print( TabToken + MyTokenSep );
    }
}

// Called only in initializerList and localLabelDecl.
void commaSep( TNode t ) {
    String s = t.getText();
    if (s.length() > 0)
        currentOutput.print( s + MyTokenSep );
    if ( t.getNextSibling() != null) {
        String x = ((TNode) t.getNextSibling()).getText();
        if ( !x.equals("}") && !x.equals(";")) {
            print( ",");
        }
    }
}

        int traceDepth = 0;
        public void reportError(RecognitionException ex) {
          if ( ex != null)   {
                System.err.println("ANTLR Tree Parsing RecognitionException Error: " + ex.getClass().getName() + " " + ex );
                ex.printStackTrace(System.err);
          }
        }
        public void reportError(NoViableAltException ex) {
                System.err.println("ANTLR Tree Parsing NoViableAltException Error: " + ex.toString());
                TNode.printTree( ex.node );
                ex.printStackTrace(System.err);
        }
        public void reportError(MismatchedTokenException ex) {
          if ( ex != null)   {
                TNode.printTree( ex.node );
                System.err.println("ANTLR Tree Parsing MismatchedTokenException Error: " + ex );
                ex.printStackTrace(System.err);
          }
        }
        public void reportError(String s) {
                System.err.println("ANTLR Error from String: " + s);
        }
        public void reportWarning(String s) {
                System.err.println("ANTLR Warning from String: " + s);
        }
        protected void match(AST t, int ttype) throws MismatchedTokenException {
                //System.out.println("match("+ttype+"); cursor is "+t);
                super.match(t, ttype);
        }
        public void match(AST t, BitSet b) throws MismatchedTokenException {
                //System.out.println("match("+b+"); cursor is "+t);
                super.match(t, b);
        }
        protected void matchNot(AST t, int ttype) throws MismatchedTokenException {
                //System.out.println("matchNot("+ttype+"); cursor is "+t);
                super.matchNot(t, ttype);
                }
        public void traceIn(String rname, AST t) {
          traceDepth += 1;
          for (int x=0; x<traceDepth; x++) System.out.print(" ");
          super.traceIn(rname, t);
        }
        public void traceOut(String rname, AST t) {
          for (int x=0; x<traceDepth; x++) System.out.print(" ");
          super.traceOut(rname, t);
          traceDepth -= 1;
        }

}

translationUnit  options {
  defaultErrorHandler=false;
}
        :
                                { initializePrinting(); }
               ( externalList )?
                                { finalizePrinting(); }
        ;



externalList
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("externalList");
       }
    }
        :  (
                ( externalDef )+
           )
  { currentOutput.print(Close); }
        ;

externalDef
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("externalDef");
       }
    }
        :  (
                declaration
        |       functionDef
        |       asm_expr
        |       typelessDeclaration
        |       s:SEMI                          { print( s ); }
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

typelessDeclaration
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("typelessDeclaration");
       }
    }
        :  (
                #(NTypeMissing initDeclList s: SEMI)    { print( s ); }
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

asm_expr
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("asm_expr");
       }
    }
        :  (
                #( a:"asm"                              { print( a ); }
                 ( v:"volatile"                         { print( v ); }
                 )?
                    lc:LCURLY                           { print( lc ); tabs++; }
                    expr
                    rc:RCURLY                           { tabs--; print( rc ); }
                    s:SEMI                              { print( s ); }
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

declaration
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("declaration");
       }
    }
        :  (
                #( NDeclaration
                    declSpecifiers
                    (
                        initDeclList
                    )?
                    ( s:SEMI { print( s ); } )+
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

declSpecifiers
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("declSpecifiers");
       }
    }
        :  (
                ( storageClassSpecifier
                | typeQualifier
                | typeSpecifier
                )+
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

storageClassSpecifier
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("storageClassSpecifier");
       }
    }
        :  (
                a:"auto"                        {  print(Open); print("storageClassSpecifier.0"); print( a ); print(Close); }
        |       b:"register"                    {  print(Open); print("storageClassSpecifier.1"); print( b ); print(Close); }
        |       c:"typedef"                     {  print(Open); print("storageClassSpecifier.2"); print( c ); print(Close); }
        |       functionStorageClassSpecifier
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

functionStorageClassSpecifier
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("functionStorageClassSpecifier");
       }
    }
        :  (
                a:"extern"                      {  print(Open); print("functionStorageClassSpecifier.0"); print( a ); print(Close); }
        |       b:"static"                      {  print(Open); print("functionStorageClassSpecifier.1"); print( b ); print(Close); }
        |       c:"inline"                      {  print(Open); print("functionStorageClassSpecifier.2"); print( c ); print(Close); }
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

typeQualifier
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("typeQualifier");
       }
    }
        :  (
                a:"const"                       {  print(Open); print("typeQualifier.0"); print( a ); print(Close); }
        |       b:"volatile"                    {  print(Open); print("typeQualifier.1"); print( b ); print(Close); }
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

typeSpecifier
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("typeSpecifier");
       }
    }
        :  (
                a:"void"                        {  print(Open); print("typeSpecifier.0"); print( a ); print(Close); }
        |       b:"char"                        {  print(Open); print("typeSpecifier.1"); print( b ); print(Close); }
        |       c:"short"                       {  print(Open); print("typeSpecifier.2"); print( c ); print(Close); }
        |       d:"int"                         {  print(Open); print("typeSpecifier.3"); print( d ); print(Close); }
        |       e:"long"                        {  print(Open); print("typeSpecifier.4"); print( e ); print(Close); }
        |       f:"float"                       {  print(Open); print("typeSpecifier.5"); print( f ); print(Close); }
        |       g:"double"                      {  print(Open); print("typeSpecifier.6"); print( g ); print(Close); }
        |       h:"signed"                      {  print(Open); print("typeSpecifier.7"); print( h ); print(Close); }
        |       i:"unsigned"                    {  print(Open); print("typeSpecifier.8"); print( i ); print(Close); }
        |       structSpecifier ( attributeDecl )*
        |       unionSpecifier  ( attributeDecl )*
        |       enumSpecifier
        |       typedefName
        |       #(n:"typeof" lp:LPAREN          {  print(Open); print("typeSpecifier.9"); print( n ); print(Close);
                                                   print(Open); print("typeSpecifier.10"); print( lp ); print(Close); }
                    ( (typeName )=> typeName
                    | expr
                    )
                    rp:RPAREN                   {  print(Open); print("typeSpecifier.11"); print( rp ); print(Close); }
                )
        |       p:"__complex"                   {  print(Open); print("typeSpecifier.12"); print( p ); print(Close); }
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

typedefName
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("typedefName");
       }
    }
        :  (
                #(NTypedefName i:ID         {  print(Open); print("typedefName.0"); print( i ); print(Close); } )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

structSpecifier
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("structSpecifier");
       }
    }
        :  (
            #( a:"struct"                       {  print(Open); print("structSpecifier.0"); print( a ); print(Close); }
                structOrUnionBody
            )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

unionSpecifier
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("unionSpecifier");
       }
    }
        :  (
            #( a:"union"                        {  print(Open); print("unionSpecifier.0"); print( a ); print(Close); }
                structOrUnionBody
            )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

structOrUnionBody
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("structOrUnionBody");
       }
    }
        :  (
                ( (ID LCURLY) => i1:ID lc1:LCURLY   {  print(Open); print("structOrUnionBody.0"); print( i1 ); print(Close); print( "{" ); tabs++; }
    // "print" left in prev line because of possible impact on Pyrex stuff.
                        ( structDeclarationList )?
                        rc1:RCURLY                  { tabs--;  print(Open); print("structOrUnionBody.1"); print( rc1 ); print(Close); }
                |   lc2:LCURLY                      {  print(Open); print("structOrUnionBody.2"); print( "{" ); print(Close); tabs++; }
                    ( structDeclarationList )?
                    rc2:RCURLY                      { tabs--;  print(Open); print("structOrUnionBody.3"); print( rc2 ); print(Close); }
                | i2:ID                     {  print(Open); print("structOrUnionBody.4"); print( i2 ); print(Close); }
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

structDeclarationList
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("structDeclarationList");
       }
    }
        :  (
                ( structDeclaration             { print( ";" ); }
                )+
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

structDeclaration
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("structDeclaration");
       }
    }
        :  (
                specifierQualifierList structDeclaratorList
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

specifierQualifierList
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("specifierQualifierList");
       }
    }
        :  (
                (
                typeSpecifier
                | typeQualifier
                )+
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

structDeclaratorList
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("structDeclaratorList");
       }
    }
        :  (
                structDeclarator
                ( { print( "," ); } structDeclarator )*
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

structDeclarator
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("structDeclarator");
       }
    }
        :  (
         
        #( NStructDeclarator
            ( declarator )?
            ( c:COLON {  print(Open); print("structDeclarator.0"); print( c ); print(Close); } expr )?
            ( attributeDecl )*
        )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

enumSpecifier
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("enumSpecifier");
       }
    }
        :  (
            #(  a:"enum"                {  print(Open); print("enumSpecifier.0"); print( a ); print(Close); }
                ( i:ID {  print(Open); print("enumSpecifier.1"); print( i ); print(Close); } )?
                ( lc:LCURLY             {  print(Open); print("enumSpecifier.2"); print( lc ); print(Close); tabs++; }
                    enumList
                  rc:RCURLY             { tabs--;  print(Open); print("enumSpecifier.3"); print( rc ); print(Close); }
                )?
            )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

enumList
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("enumList");
       }
    }
        :  (
         
                enumerator ( { print( "," ); } enumerator)*
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

enumerator
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("enumerator");
       }
    }
        :  (
                i:ID            {  print(Open); print("enumerator.0"); print( i ); print(Close); }
                ( b:ASSIGN      {  print(Open); print("enumerator.1"); print( b ); print(Close); }
                  expr
                )?
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

attributeDecl
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("attributeDecl");
       }
    }
        :  (
         
        #( a:"__attribute"            {  print(Open); print("attributeDecl.0"); print( a ); print(Close); }
           (b:. { // Processes strings without using stringConst.
                  print(Open); print("attributeDecl.1"); print( b );
                  print(Close); } )*
        )
        | #( n:NAsmAttribute          { print( n ); }
             lp:LPAREN                { print( lp ); }
             expr
             rp:RPAREN                { print( rp ); }
           )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

initDeclList
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("initDeclList");
       }
    }
        :  (
                initDecl
                ( { print( "," ); } initDecl )*
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

initDecl
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("initDecl");
       }
    String declName = ""; }
        :  (
                #(NInitDecl
                declarator
                ( attributeDecl )*
                ( a:ASSIGN              {  print(Open); print("initDecl.0"); print( a ); print(Close); }
                  initializer
                | b:COLON               {  print(Open); print("initDecl.1"); print( b ); print(Close); }
                  expr
                )?
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

pointerGroup
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("pointerGroup");
       }
    }
        :  (
                #( NPointerGroup
                   ( a:STAR             {  print(Open); print("pointerGroup.0"); print( a ); print(Close); }
                    ( typeQualifier )*
                   )+
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

idList
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("idList");
       }
    }
        :  (
                i:ID                   {  print(Open); print("idList.0"); print( i ); print(Close); }
                (  c:COMMA             {  print( c ); }
                   id:ID               {  print(Open); print("idList.1"); print( id ); print(Close); }
                )*
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

initializer
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("initializer");
       }
    }
        :  (
                #( NInitializer (initializerElementLabel)? expr )
                | lcurlyInitializer
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

initializerElementLabel
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("initializerElementLabel");
       }
    }
        :  (
            #( NInitializerElementLabel
                (
                    ( l:LBRACKET              {  print(Open); print("initializerElementLabel.0"); print( l ); print(Close); }
                        expr
                        r:RBRACKET            {  print(Open); print("initializerElementLabel.1"); print( r ); print(Close); }
                        (a1:ASSIGN            {  print(Open); print("initializerElementLabel.2"); print( a1 ); print(Close); } )?
                    )
                    | i1:ID c:COLON           {  print(Open); print("initializerElementLabel.3"); print( i1 ); print(Close);
                                                 print(Open); print("initializerElementLabel.4"); print( c ); print(Close); }
                    | d:DOT i2:ID a2:ASSIGN   {  print(Open); print("initializerElementLabel.5"); print( d ); print(Close);
                                                 print(Open); print("initializerElementLabel.6"); print( i2 ); print(Close);
                                                 print(Open); print("initializerElementLabel.7"); print( a2 ); print(Close); }
                )
            )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

lcurlyInitializer
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("lcurlyInitializer");
       }
    }
        :  (
              #(n:NLcurlyInitializer    {  print(Open); print("lcurlyInitializer.0"); print( n ); print(Close); tabs++; }
                initializerList
                rc:RCURLY               { tabs--;  print(Open); print("lcurlyInitializer.1"); print( rc ); print(Close); }
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

initializerList
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("initializerList");
       }
    }
        :  (
                ( i:initializer { commaSep( i ); }
                )*
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

declarator
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("declarator");
       }
    }
        :  (
            #( NDeclarator
                ( pointerGroup )?
                ( id:ID                         {  print(Open); print("declarator.0"); print( id ); print(Close); }
                | lp:LPAREN {  print(Open); print("declarator.1"); print( lp ); print(Close); } declarator rp:RPAREN {  print(Open); print("declarator.2"); print( rp ); print(Close); }
                )
                (   #( n:NParameterTypeList       {  print(Open); print("declarator.3"); print( n ); print(Close); }
                    (
                        parameterTypeList
                        | (idList)?
                    )
                    r:RPAREN                      {  print(Open); print("declarator.4"); print( r ); print(Close); }
                    )
                 | lb:LBRACKET {  print(Open); print("declarator.5"); print( lb ); print(Close); } ( expr )? rb:RBRACKET {  print(Open); print("declarator.6"); print( rb ); print(Close); }
                )*
             )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

parameterTypeList
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("parameterTypeList");
       }
    }
        :  (
                ( parameterDeclaration
                    ( c:COMMA { print( c ); }
                      | s:SEMI { print( s ); }
                    )?
                )+
                ( v:VARARGS {  print(Open); print("parameterTypeList.0"); print( v ); print(Close); } )?
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

parameterDeclaration
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("parameterDeclaration");
       }
    }
        :  (
                #( NParameterDeclaration
                declSpecifiers
                (declarator | nonemptyAbstractDeclarator)?
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

functionDef
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("functionDef");
       }
    }
        :  (
            #( NFunctionDef
                ( functionDeclSpecifiers)?
                declarator
                (declaration
                 | v:VARARGS    {  print(Open); print("functionDef.0"); print( v ); print(Close); }
                )*
                compoundStatement
            )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

functionDeclSpecifiers
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("functionDeclSpecifiers");
       }
    }
        :  (
         
                ( functionStorageClassSpecifier
                | typeQualifier
                | typeSpecifier
                )+
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

declarationList
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("declarationList");
       }
    }
        :  (
         
                (   //ANTLR doesn't know that declarationList properly eats
                    //all the declarations so it warns about the ambiguity
                    options {
                        warnWhenFollowAmbig = false;
                    } :
                localLabelDecl
                |  declaration
                )+
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

localLabelDecl
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("localLabelDecl");
       }
    }
        :  (
            #(a:"__label__"             {  print(Open); print("localLabelDecl.0"); print( a ); print(Close); }
              ( i:ID                    { commaSep( i ); }
              )+
                                        { print( ";" ); }
            )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

compoundStatement
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("compoundStatement");
       }
    }
        :  (
                #( cs:NCompoundStatement         {  print(Open); print("compoundStatement.0"); print( cs ); print(Close); tabs++; }
                ( declarationList
                | functionDef
                )*
                ( statementList )?
                rc:RCURLY                        { tabs--;  print(Open); print("compoundStatement.1"); print( rc ); print(Close); }
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

statementList
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("statementList");
       }
    }
        :  (
                ( statement )+
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

statement
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("statement");
       }
    }
        :  (
                statementBody
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

statementBody
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("statementBody");
       }
    }
        :  (
                s:SEMI                  { print( s ); }
        |       compoundStatement       // Group of statements
        |       #(NStatementExpr
                expr                    { print( ";" ); }
                )                    // Expressions
// Iteration statements:
        |       #( w:"while"       {  print(Open); print("statementBody.0"); print( w ); print(Close);
                                      print(Open); print("statementBody.1"); print( "(" ); print(Close); }
                expr               {  print(Open); print("statementBody.2"); print( ")" ); print(Close); }
                statement )
        |       #( d:"do"          {  print(Open); print("statementBody.3"); print( d ); print(Close); }
                statement
                                   {  print(Open); print("statementBody.4"); print( "while" ); print(Close);
                                      print(Open); print("statementBody.5"); print( "(" ); print(Close); }
                expr
                                   {  print(Open); print("statementBody.6"); print( ")" ); print(Close); print( ";" ); }
                )
        |       #( f:"for"         {  print(Open); print("statementBody.7"); print( f ); print(Close);
                                      print(Open); print("statementBody.8"); print( "(" ); print(Close); }
                expr               { print( ";" ); }
                expr               { print( ";" ); }
                expr               {  print(Open); print("statementBody.9"); print( ")" ); print(Close); }
                statement
                )
// Jump statements:
        |       #( g:"goto"        {  print(Open); print("statementBody.10"); print( g ); print(Close); }
                   expr            { print( ";" ); }
                )
        |       c:"continue"       {  print(Open); print("statementBody.11"); print( c ); print(Close); print( ";" ); }
        |       b:"break"          {  print(Open); print("statementBody.12"); print( b ); print(Close); print( ";" ); }
        |       #( r:"return"      {  print(Open); print("statementBody.13"); print( r ); print(Close); }
                ( expr )?
                                   { print( ";" ); }
                )
// Labeled statements:
        |       #( NLabel
                ni:ID              {  print(Open); print("statementBody.14"); print( ni ); print(Close);
                                      print(Open); print("statementBody.15"); print( ":" ); print(Close); }
                ( statement )?
                )
        |       #(
                ca:"case"          {  print(Open); print("statementBody.16"); print( ca ); print(Close); }
                expr               {  print(Open); print("statementBody.17"); print( ":" ); print(Close); }
                (statement)?
                )
        |       #(
                de:"default"       {  print(Open); print("statementBody.18"); print( de ); print(Close);
                                      print(Open); print("statementBody.19"); print( ":" ); print(Close); }
                (statement)?
                )
// Selection statements:
        |       #( i:"if"          {  print(Open); print("statementBody.20"); print( i ); print(Close);
                                      print(Open); print("statementBody.21"); print( "(" ); print(Close); }
                 expr              {  print(Open); print("statementBody.22"); print( ")" ); print(Close); }
                statement
                (   e:"else"       {  print(Open); print("statementBody.23"); print( e ); print(Close); }
                    statement
                )?
                )
        |       #( sw:"switch"     {  print(Open); print("statementBody.24"); print( sw ); print(Close);
                                      print(Open); print("statementBody.25"); print( "(" ); print(Close); }
                expr               {  print(Open); print("statementBody.26"); print( ")" ); print(Close); }
                statement
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

expr
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("expr");
       }
    }
        :  (
         
                binaryExpr
        |       conditionalExpr
        |       castExpr
        |       unaryExpr
        |       postfixExpr
        |       primaryExpr
        |       emptyExpr
        |       compoundStatementExpr
        |       initializer
        |       rangeExpr
        |       gnuAsmExpr
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

emptyExpr
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("emptyExpr");
       }
    }
        :  (
            NEmptyExpression
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

compoundStatementExpr
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("compoundStatementExpr");
       }
    }
        :  (
            #(l:LPAREN                  {  print(Open); print("compoundStatementExpr.0"); print( l ); print(Close); }
                compoundStatement
                r:RPAREN                {  print(Open); print("compoundStatementExpr.1"); print( r ); print(Close); }
            )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

rangeExpr
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("rangeExpr");
       }
    }
        :  (
            #(NRangeExpr expr v:VARARGS{  print(Open); print("rangeExpr.0"); print( v ); print(Close); } expr)
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

gnuAsmExpr
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("gnuAsmExpr");
       }
    }
        :  (
            #(n:NGnuAsmExpr                        { print( n ); }
                (v:"volatile" { print( v ); } )?
                lp:LPAREN                          { print( lp ); }
                stringConst
                (  options { warnWhenFollowAmbig = false; }:
                    c1:COLON                       { print( c1 ); }
                    (strOptExprPair
                        ( c2:COMMA { print( c2 ); } strOptExprPair)*
                    )?
                  (  options { warnWhenFollowAmbig = false; }:
                    c3:COLON                       { print( c3 ); }
                      (strOptExprPair
                        ( c4:COMMA { print( c4 ); } strOptExprPair)*
                      )?
                  )?
                )?
                ( c5:COLON                         { print( c5 ); }
                  stringConst
                  ( c6:COMMA                       { print( c6 ); }
                    stringConst
                  )*
                )?
                rp:RPAREN                          { print( rp ); }
            )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

strOptExprPair
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("strOptExprPair");
       }
    }
        :  (
            stringConst
            (
            l:LPAREN                    {  print(Open); print("strOptExprPair.0"); print( l ); print(Close); }
            expr
            r:RPAREN                    {  print(Open); print("strOptExprPair.1"); print( r ); print(Close); }
            )?
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

binaryOperator
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("binaryOperator");
       }
    }
        :  (
                ASSIGN
        |       DIV_ASSIGN
        |       PLUS_ASSIGN
        |       MINUS_ASSIGN
        |       STAR_ASSIGN
        |       MOD_ASSIGN
        |       RSHIFT_ASSIGN
        |       LSHIFT_ASSIGN
        |       BAND_ASSIGN
        |       BOR_ASSIGN
        |       BXOR_ASSIGN
        |       LOR
        |       LAND
        |       BOR
        |       BXOR
        |       BAND
        |       EQUAL
        |       NOT_EQUAL
        |       LT
        |       LTE
        |       GT
        |       GTE
        |       LSHIFT
        |       RSHIFT
        |       PLUS
        |       MINUS
        |       STAR
        |       DIV
        |       MOD
        |       NCommaExpr
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

binaryExpr
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("binaryExpr");
       }
    }
        :  (
                b:binaryOperator
                    // no rules allowed as roots, so here I manually get
                    // the first and second children of the binary operator
                    // and then print them out in the right order
                                        {       TNode e1, e2;
                                                e1 = (TNode) b.getFirstChild();
                                                e2 = (TNode) e1.getNextSibling();
                                                expr( e1 );
                                                 print(Open); print("binaryExpr.0"); print( b ); print(Close);
                                                expr( e2 );
                                        }
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

conditionalExpr
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("conditionalExpr");
       }
    }
        :  (
                #( q:QUESTION
                expr                    {  print(Open); print("conditionalExpr.0"); print( q ); print(Close); }
                ( expr )?
                c:COLON                 {  print(Open); print("conditionalExpr.1"); print( c ); print(Close); }
                expr
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

castExpr
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("castExpr");
       }
    }
        :  (
                #(
                c:NCast                 {  print(Open); print("castExpr.0"); print( c ); print(Close); }
                typeName
                rp:RPAREN               {  print(Open); print("castExpr.1"); print( rp ); print(Close); }
                expr
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

typeName
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("typeName");
       }
    }
        :  (
                specifierQualifierList (nonemptyAbstractDeclarator)?
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

nonemptyAbstractDeclarator
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("nonemptyAbstractDeclarator");
       }
    }
        :  (
            #( NNonemptyAbstractDeclarator
            (   pointerGroup
                (   (lp1:LPAREN                         {  print(Open); print("nonemptyAbstractDeclarator.0"); print( lp1 ); print(Close); }
                    (   nonemptyAbstractDeclarator
                        | parameterTypeList
                    )?
                    rp1:RPAREN                          {  print(Open); print("nonemptyAbstractDeclarator.1"); print( rp1 ); print(Close); }
                    )
                | (
                    lb1:LBRACKET                        {  print(Open); print("nonemptyAbstractDeclarator.2"); print( lb1 ); print(Close); }
                    (expr)?
                    rb1:RBRACKET                        {  print(Open); print("nonemptyAbstractDeclarator.3"); print( rb1 ); print(Close); }
                  )
                )*
            |  (   (lp2:LPAREN                          {  print(Open); print("nonemptyAbstractDeclarator.4"); print( lp2 ); print(Close); }
                    (   nonemptyAbstractDeclarator
                        | parameterTypeList
                    )?
                    rp2:RPAREN                          {  print(Open); print("nonemptyAbstractDeclarator.5"); print( rp2 ); print(Close); }
                   )
                | (
                    lb2:LBRACKET                        {  print(Open); print("nonemptyAbstractDeclarator.6"); print( lb2 ); print(Close); }
                    (expr)?
                    rb2:RBRACKET                        {  print(Open); print("nonemptyAbstractDeclarator.7"); print( rb2 ); print(Close); }
                  )
                )+
            )
            )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

unaryExpr
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("unaryExpr");
       }
    }
        :  (
                #( i:INC {  print(Open); print("unaryExpr.0"); print( i ); print(Close); } expr )
        |       #( d:DEC {  print(Open); print("unaryExpr.1"); print( d ); print(Close); } expr )
        |       #( NUnaryExpr u:unaryOperator {  print(Open); print("unaryExpr.2"); print( u ); print(Close); } expr)
        |       #( s:"sizeof"                           {  print(Open); print("unaryExpr.3"); print( s ); print(Close); }
                    ( ( LPAREN typeName )=>
                        lps:LPAREN                      {  print(Open); print("unaryExpr.4"); print( lps ); print(Close); }
                        typeName
                        rps:RPAREN                      {  print(Open); print("unaryExpr.5"); print( rps ); print(Close); }
                    | expr
                    )
                )
        |       #( a:"__alignof"                             {  print(Open); print("unaryExpr.6"); print( a ); print(Close); }
                    ( ( LPAREN typeName )=>
                        lpa:LPAREN                      {  print(Open); print("unaryExpr.7"); print( lpa ); print(Close); }
                        typeName
                        rpa:RPAREN                      {  print(Open); print("unaryExpr.8"); print( rpa ); print(Close); }
                    | expr
                    )
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

unaryOperator
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("unaryOperator");
       }
    }
        :  (
                BAND
        |       STAR
        |       PLUS
        |       MINUS
        |       BNOT
        |       LNOT
        |       LAND
        |       "__real"
        |       "__imag"
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

postfixExpr
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("postfixExpr");
       }
    }
        :  (
                #( NPostfixExpr
                    primaryExpr
                    ( a:PTR b:ID               {  print(Open); print("postfixExpr.0"); print( a ); print(Close); 
                                                  print(Open); print("postfixExpr.1"); print( b ); print(Close); }
                    | c:DOT d:ID               {  print(Open); print("postfixExpr.2"); print( c ); print(Close);
                                                  print(Open); print("postfixExpr.3"); print( d ); print(Close); }
                    | #( n:NFunctionCallArgs   {  print(Open); print("postfixExpr.4"); print( n ); print(Close); }
                        (argExprList)?
                        rp:RPAREN              {  print(Open); print("postfixExpr.5"); print( rp ); print(Close); }
                        )
                    | lb:LBRACKET              {  print(Open); print("postfixExpr.6"); print( lb ); print(Close); }
                        expr
                        rb:RBRACKET            {  print(Open); print("postfixExpr.7"); print( rb ); print(Close); }
                    | f:INC                    {  print(Open); print("postfixExpr.8"); print( f ); print(Close); }
                    | g:DEC                    {  print(Open); print("postfixExpr.9"); print( g ); print(Close); }
                    )+
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

primaryExpr
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("primaryExpr");
       }
    }
        :  (
                i:ID                            {  print(Open); print("primaryExpr.0"); print( i ); print(Close); }
        |       n:Number                        {  print(Open); print("primaryExpr.1"); print( n ); print(Close); }
        |       charConst
        |       stringConst
// JTC:
// ID should catch the enumerator
// leaving it in gives ambiguous err
//      | enumerator
        |       #( eg:NExpressionGroup          {  print(Open); print("primaryExpr.2"); print( eg ); print(Close); }
                 expr                           {  print(Open); print("primaryExpr.3"); print( ")" ); print(Close); }
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

argExprList
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("argExprList");
       }
    }
        :  (
                expr ( { print( "," ); } expr )*
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

protected
charConst
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("charConst");
       }
    }
        :  (
                c:CharLiteral                   {  print(Open); print("charConst.0"); print( c ); print(Close); }
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

protected
stringConst
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("stringConst");
       }
    }
        :  (
                #( NStringSeq
                    (
                    s:StringLiteral             {  print(Open); print("stringConst.0"); print( s );
                print(Close); }
                    )+
                )
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

protected
intConst
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("intConst");
       }
    }
        :  (
                IntOctalConst
        |       LongOctalConst
        |       UnsignedOctalConst
        |       IntIntConst
        |       LongIntConst
        |       UnsignedIntConst
        |       IntHexConst
        |       LongHexConst
        |       UnsignedHexConst
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

protected
floatConst
  { if ( inputState.guessing==0 ) {
          print(Open);
          print("floatConst");
       }
    }
        :  (
                FloatDoubleConst
        |       DoubleDoubleConst
        |       LongDoubleConst
           )
  { currentOutput.print(Close + MyTokenSep); }
        ;

