import traceback, textwrap, inspect, sys

def LN(*args, **kwargs):
    """Prints a line number and some text.

    A variable number of positional arguments are allowed. If
        LN(obj0, obj1, obj2)
    is called, the text part of the output looks like the output from
        print obj0, obj1, obj2
    The optional keyword "wrap" causes the message to be line-wrapped. The
    argument to "wrap" should be "1" or "True". "name" is another optional
    keyword parameter which is best explained by an example:
        from linenum import LN
        def fun1():
            print LN('error', 'is', 'here')
        def fun2():
            print LN('error',  'is', 'here', name='mess')
        fun1()
        fun2()
    The output is:
        L3 fun1: error is here
        L5 mess: error is here
    """
    stack = traceback.extract_stack()
    path, lineno, funcname, source_line = stack[-2]
    modulename = inspect.getmodulename(path)
    out = []
    for obj in args:
        out.append(str(obj))

    text = ' '.join(out)
    if 'name' in kwargs:
        name = kwargs['name']
    else:
        name = '%s, %s' % (modulename, funcname)
    text = 'L%s %s: %s' % (lineno, name, text)
    if 'wrap' in kwargs and kwargs['wrap']:
        text = textwrap.fill(text)
    print >> sys.stdout, text

def MLLN(*args, **kwargs):
    """Prints a line number and some text.

    A variable number of positional arguments are allowed. If
        LN(obj0, obj1, obj2)
    is called, the text part of the output looks like the output from
        print obj0, obj1, obj2
    The optional keyword "wrap" causes the message to be line-wrapped. The
    argument to "wrap" should be "1" or "True". "name" is another optional
    keyword parameter which is best explained by an example:
        from linenum import LN
        def fun1():
            print LN('error', 'is', 'here')
        def fun2():
            print LN('error',  'is', 'here', name='mess')
        fun1()
        fun2()
    The output is:
        L3 fun1: error is here
        L5 mess: error is here
    """
    stack = traceback.extract_stack()
    path, lineno, funcname, source_line = stack[-2]
    modulename = inspect.getmodulename(path)
    out = []
    for obj in args:
        text = str(obj)
        lines = text.splitlines()
        lines.append('')
        text = '  ' + '\n  '.join(lines)
        out.append(text)
    text = '\n'.join(out)
    if 'name' in kwargs:
        name = kwargs['name']
    else:
        name = '%s, %s' % (modulename, funcname)
    text = 'L%s %s:\n%s' % (lineno, name, text)
    if 'wrap' in kwargs and kwargs['wrap']:
        text = textwrap.fill(text)
    print >> sys.stdout, text
