import java.io.*;

import antlr.*;

// A slightly modified version of "TestThrough.java".

public class UnparsePlus
{
    public static void main(String[] args)
    {
        PrintStream syserr = System.err;
        PrintStream sysout = System.out;
        try
            {
            String programName = args[0];
            String outFile = args[1];
            String errFile = args[2];

            PrintStream newout = new PrintStream(new FileOutputStream(outFile));
            System.setOut(newout);
            PrintStream newerr = new PrintStream(new FileOutputStream(errFile));
            System.setErr(newerr);

            String originalSource = "";
            DataInputStream dis = null;
            if (programName.equals("-")) {
                dis = new DataInputStream( System.in );
            }   
            else {
                dis = new DataInputStream(new FileInputStream(programName));
            }
            GnuCLexer lexer =
                new GnuCLexer ( dis );
            lexer.setTokenObjectClass("CToken");
            lexer.initialize();
            // Parse the input expression.
            GnuCParser parser = new GnuCParser ( lexer );
            
            // set AST node type to TNode or get nasty cast class errors
            parser.setASTNodeType(TNode.class.getName());
            TNode.setTokenVocabulary("GNUCTokenTypes");

            // invoke parser
            try {
                parser.translationUnit();
            }
            catch (RecognitionException e) {
                System.err.println("Fatal IO error:\n"+e);
                System.exit(1);
            }
            catch (TokenStreamException e) {
                System.err.println("Fatal IO error:\n"+e);
                System.exit(1);
            }

            // Garbage collection hint
            System.gc();

            GnuCEmitter_print e = new GnuCEmitter_print(lexer.getPreprocessorInfoChannel());
            // set AST node type to TNode or get nasty cast class errors
            e.setASTNodeType(TNode.class.getName());

            // walk that tree
            e.translationUnit( parser.getAST() );
            System.setOut(sysout);
            System.setErr(syserr);
            }
        catch ( Exception e )
            {
            System.err.println ( "exception: " + e);
            e.printStackTrace();
            System.setOut(sysout);
            System.setErr(syserr);
            }
    }
}
