#define CV_MAX_DIM 40
/******************************************************************/
/* cvtypes.h */

typedef unsigned int size_t;
typedef unsigned char uchar;
typedef void CvArr;
typedef struct _IplImage {
    int  nSize;
    int  ID;
    int  nChannels;
    int  depth;
    int  dataOrder;
    int  origin;
    int  width;
    int  height;
    struct _IplROI *roi;
    int  imageSize;
    char* imageData;
    int  widthStep;
    char* imageDataOrigin;
}
IplImage;
typedef struct _IplTileInfo IplTileInfo;
typedef struct _IplROI {
    int  coi;
    int  xOffset;
    int  yOffset;
    int  width;
    int  height;
}
IplROI;
typedef struct _IplConvKernel
{
    int  nCols;
    int  nRows;
    int  anchorX;
    int  anchorY;
    int *values;
    int  nShiftR;
}
IplConvKernel;
typedef struct _IplConvKernelFP
{
    int  nCols;
    int  nRows;
    int  anchorX;
    int  anchorY;
    float *values;
}
IplConvKernelFP;
typedef union {
    uchar* ptr;
    short* s;
    int* i;
    float* fl;
    double* db;
} CvMat_data;
typedef struct CvMat
{
    int type;
    int step;
    int* refcount;
    CvMat_data data;
    int rows;
    int cols;
} CvMat;
CvMat cvMat( int rows, int cols, int type, void* data );
double  cvmGet( const CvMat* mat, int i, int j );
void  cvmSet( CvMat* mat, int i, int j, double val );
typedef struct {
    int size;
    int step;
} CvMatND_dim;
typedef struct CvMatND
{
    int type;
    int dims;
    int* refcount;
    CvMat_data data;
    CvMatND_dim dim[CV_MAX_DIM];
}
CvMatND;
struct CvSet;
typedef struct CvSparseMat
{
    int type;
    int dims;
    int* refcount;
    struct CvSet* heap;
    void** hashtable;
    int hashsize;
    int total;
    int valoffset;
    int idxoffset;
    int size[CV_MAX_DIM];
}
CvSparseMat;
typedef struct CvSparseNode
{
    unsigned hashval;
    struct CvSparseNode* next;
}
CvSparseNode;
typedef struct CvSparseMatIterator
{
    CvSparseMat* mat;
    CvSparseNode* node;
    int curidx;
}
CvSparseMatIterator;
typedef struct CvHistogram
{
    int     type;
    CvArr*  bins;
    float   thresh[CV_MAX_DIM][2];
    float** thresh2;
    CvMatND mat;
}
CvHistogram;
int  cvRound( double val );
int  cvFloor( double val );
int  cvCeil( double val );
typedef struct CvRect
{
    int x;
    int y;
    int width;
    int height;
}
CvRect;
CvRect  cvRect( int x, int y, int width, int height );
IplROI  cvRectToROI( CvRect rect, int coi );
CvRect  cvROIToRect( IplROI roi );
typedef struct CvTermCriteria
{
    int    type;
    int    maxIter;
    double epsilon;
}
CvTermCriteria;
CvTermCriteria  cvTermCriteria( int type, int maxIter, double epsilon );
typedef struct CvPoint
{
    int x;
    int y;
}
CvPoint;
CvPoint  cvPoint( int x, int y );
typedef struct CvPoint2D32f
{
    float x;
    float y;
}
CvPoint2D32f;
CvPoint2D32f  cvPoint2D32f( double x, double y );
CvPoint2D32f  cvPointTo32f( CvPoint pt );
CvPoint  cvPointFrom32f( CvPoint2D32f pt );
typedef struct CvPoint3D32f
{
    float x;
    float y;
    float z;
}
CvPoint3D32f;
CvPoint3D32f  cvPoint3D32f( double x, double y, double z );
typedef struct CvPoint2D64d
{
    double x;
    double y;
}
CvPoint2D64d;
typedef struct CvPoint3D64d
{
    double x;
    double y;
    double z;
}
CvPoint3D64d;
typedef struct
{
    int width;
    int height;
}
CvSize;
CvSize  cvSize( int width, int height );
typedef struct CvSize2D32f
{
    float width;
    float height;
}
CvSize2D32f;
CvSize2D32f  cvSize2D32f( double width, double height );
typedef struct CvBox2D
{
    CvPoint2D32f center;
    CvSize2D32f  size;
    float angle;
}
CvBox2D;
typedef struct CvSlice
{
    int  startIndex, endIndex;
} CvSlice;
CvSlice  cvSlice( int start, int end );
typedef struct CvScalar
{
    double val[4];
}
CvScalar;
CvScalar  cvScalar( double a, double b ,
                               double c , double d );
CvScalar  cvRealScalar( double a );
CvScalar  cvScalarAll( double a );
struct CvSeq;
typedef struct CvConnectedComp
{
    double area;
    double value;
    CvRect rect;
    struct CvSeq* contour;
}
CvConnectedComp;
float  cvInvSqrt( float arg );
float  cvSqrt( float arg );
typedef enum CvDisType
{
    CV_DIST_USER         = -1,
    CV_DIST_L1           = 1,
    CV_DIST_L2           = 2,
    CV_DIST_C            = 3,
    CV_DIST_L12          = 4,
    CV_DIST_FAIR         = 5,
    CV_DIST_WELSCH       = 6,
    CV_DIST_HUBER        = 7
}
CvDisType;
typedef enum CvFilter
{
    CV_GAUSSIAN_5x5 = 7
}
CvFilter;
typedef struct CvMemBlock
{
    struct CvMemBlock*  prev;
    struct CvMemBlock*  next;
}
CvMemBlock;
typedef struct CvMemStorage
{
    int     signature;
    CvMemBlock* bottom;
    CvMemBlock* top;
    struct  CvMemStorage* parent;
    int     block_size;
    int     free_space;
}
CvMemStorage;
typedef struct CvMemStoragePos
{
    CvMemBlock* top;
    int  free_space;
}
CvMemStoragePos;
typedef struct CvSeqBlock
{
    struct CvSeqBlock*  prev;
    struct CvSeqBlock*  next;
    int    start_index;
    int    count;
    char*  data;
} CvSeqBlock;
typedef struct CvSeq
{
/*  CV_SEQUENCE_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
}
CvSeq;
typedef struct CvSetElem
{
/*  CV_SET_ELEM_FIELDS(CvSetElem) */
    int  flags;
    struct CvSetElem* next_free;
}
CvSetElem;
typedef struct CvSet
{
/*  CV_SET_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvSetElem* free_elems;
}
CvSet;
typedef struct CvGraphEdge
{
/*  CV_GRAPH_EDGE_FIELDS() */
    int flags;
    float weight;
    struct CvGraphEdge* next[2];
    struct CvGraphVtx* vtx[2];
}
CvGraphEdge;
typedef struct CvGraphVtx
{
/*  CV_GRAPH_VERTEX_FIELDS() */
    int flags;
    struct CvGraphEdge* first;
}
CvGraphVtx;
typedef struct CvGraphVtx2D
{
/*  CV_GRAPH_VERTEX_FIELDS() */
    int flags;
    struct CvGraphEdge* first;
    CvPoint2D32f* ptr;
}
CvGraphVtx2D;
typedef struct CvGraph
{
/*  CV_GRAPH_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvSetElem* free_elems;
    CvSet* edges;
}
CvGraph;
typedef struct CvChain
{
/*  CV_SEQUENCE_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvPoint  origin;
}
CvChain;
typedef struct CvContour
{
/*  CV_CONTOUR_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvRect rect;
    int color;
    int reserved[3];
}
CvContour;
typedef struct CvSeqWriter
{
/*  CV_SEQ_WRITER_FIELDS() */
    int          header_size;
    CvSeq*       seq;
    CvSeqBlock*  block;
    char*        ptr;
    char*        block_min;
    char*        block_max;
    int  reserved[4];
}
CvSeqWriter;
typedef struct CvSeqReader
{
/*  CV_SEQ_READER_FIELDS() */
    int          header_size;
    CvSeq*       seq;
    CvSeqBlock*  block;
    char*        ptr;
    char*        block_min;
    char*        block_max;
    int          delta_index;
    char*        prev_elem;
    int  reserved[4];
}
CvSeqReader;
typedef struct
{
    const char* file;
    int         line;
}
CvStackRecord;
typedef float*   CvVect32f;
typedef float*   CvMatr32f;
typedef double*  CvVect64d;
typedef double*  CvMatr64d;
typedef struct CvMatrix3
{
    float m[3][3];
}
CvMatrix3;
typedef enum CvStatus
{
    CV_INPLACE_NOT_SUPPORTED_ERR= -112,
    CV_UNMATCHED_ROI_ERR        = -111,
    CV_NOTFOUND_ERR             = -110,
    CV_BADCONVERGENCE_ERR       = -109,
    CV_BADDEPTH_ERR             = -107,
    CV_BADROI_ERR               = -106,
    CV_BADHEADER_ERR            = -105,
    CV_UNMATCHED_FORMATS_ERR    = -104,
    CV_UNSUPPORTED_COI_ERR      = -103,
    CV_UNSUPPORTED_CHANNELS_ERR = -102,
    CV_UNSUPPORTED_DEPTH_ERR    = -101,
    CV_UNSUPPORTED_FORMAT_ERR   = -100,
    CV_BADARG_ERR      = -49,
    CV_NOTDEFINED_ERR  = -48,
    CV_BADCHANNELS_ERR = -47,
    CV_BADRANGE_ERR    = -44,
    CV_BADSTEP_ERR     = -29,
    CV_BADFLAG_ERR     =  -12,
    CV_DIV_BY_ZERO_ERR =  -11,
    CV_BADCOEF_ERR     =  -10,
    CV_BADFACTOR_ERR   =  -7,
    CV_BADPOINT_ERR    =  -6,
    CV_BADSCALE_ERR    =  -4,
    CV_OUTOFMEM_ERR    =  -3,
    CV_NULLPTR_ERR     =  -2,
    CV_BADSIZE_ERR     =  -1,
    CV_NO_ERR          =   0,
    CV_OK              =   CV_NO_ERR
}
CvStatus;
/******************************************************************/
/*cverror.h */
typedef int CVStatus;
/******************************************************************/
/* cv.h */
void*  cvAlloc( size_t size );
void   cvFree( void** ptr );
IplImage*  cvCreateImageHeader( CvSize size, int depth, int channels );
IplImage* cvInitImageHeader( IplImage* image, CvSize size, int depth,
                                       int channels, int origin ,
                                       int align );
IplImage*  cvCreateImage( CvSize size, int depth, int channels );
void  cvReleaseImageHeader( IplImage** image );
void  cvReleaseImage( IplImage** image );
IplImage* cvCloneImage( const IplImage* image );
void  cvSetImageCOI( IplImage* image, int coi );
int  cvGetImageCOI( IplImage* image );
void  cvSetImageROI( IplImage* image, CvRect rect );
void  cvResetImageROI( IplImage* image );
CvRect cvGetImageROI( const IplImage* image );
CvMat*  cvCreateMatHeader( int rows, int cols, int type );
CvMat* cvInitMatHeader( CvMat* mat, int rows, int cols,
                                  int type, void* data ,
                                  int step );
CvMat*  cvCreateMat( int rows, int cols, int type );
void  cvReleaseMat( CvMat** mat );
void  cvDecRefData( CvArr* arr );
int  cvIncRefData( CvArr* arr );
CvMat* cvCloneMat( const CvMat* mat );
CvMat* cvGetSubRect( const CvArr* arr, CvMat* submat, CvRect rect );
CvMat* cvGetRows( const CvArr* arr, CvMat* submat,
                            int start_row, int end_row );
void  cvGetRow( const CvArr* arr, CvMat* submat, int row );
CvMat* cvGetCols( const CvArr* arr, CvMat* submat,
                               int start_col, int end_col );
void  cvGetCol( const CvArr* arr, CvMat* submat, int col );
CvMat* cvGetDiag( const CvArr* arr, CvMat* submat,
                            int diag );
void cvScalarToRawData( const CvScalar* scalar, void* data, int type,
                                  int extend_to_12  );
void cvRawDataToScalar( const void* data, int type, CvScalar* scalar );
CvMatND*  cvCreateMatNDHeader( int dims, const int* sizes, int type );
CvMatND*  cvCreateMatND( int dims, const int* sizes, int type );
CvMatND*  cvInitMatNDHeader( CvMatND* mat, int dims, const int* sizes,
                                        int type, void* data  );
void  cvReleaseMatND( CvMatND** mat );
CvMatND* cvCloneMatND( const CvMatND* mat );
CvSparseMat*  cvCreateSparseMat( int dims, const int* sizes, int type );
void  cvReleaseSparseMat( CvSparseMat** mat );
CvSparseMat* cvCloneSparseMat( const CvSparseMat* mat );
CvSparseNode* cvInitSparseMatIterator( const CvSparseMat* mat,
                                                  CvSparseMatIterator* matIterator );
CvSparseNode* cvGetNextSparseNode( CvSparseMatIterator* matIterator );
int cvGetElemType( const CvArr* arr );
int cvGetDims( const CvArr* arr, int* sizes  );
int cvGetDimSize( const CvArr* arr, int index );
uchar* cvPtr1D( const CvArr* arr, int idx1, int* type );
uchar* cvPtr2D( const CvArr* arr, int idx1, int idx2, int* type  );
uchar* cvPtr3D( const CvArr* arr, int idx1, int idx2, int idx3,
                          int* type );
uchar* cvPtrND( const CvArr* arr, int* idx, int* type  );
CvScalar cvGet1D( const CvArr* arr, int idx1 );
CvScalar cvGet2D( const CvArr* arr, int idx1, int idx2 );
CvScalar cvGet3D( const CvArr* arr, int idx1, int idx2, int idx3 );
CvScalar cvGetND( const CvArr* arr, int* idx );
double cvGetReal1D( const CvArr* arr, int idx1 );
double cvGetReal2D( const CvArr* arr, int idx1, int idx2 );
double cvGetReal3D( const CvArr* arr, int idx1, int idx2, int idx3 );
double cvGetRealND( const CvArr* arr, int* idx );
void cvSet1D( CvArr* arr, int idx1, CvScalar value );
void cvSet2D( CvArr* arr, int idx1, int idx2, CvScalar value );
void cvSet3D( CvArr* arr, int idx1, int idx2, int idx3, CvScalar value );
void cvSetND( CvArr* arr, int* idx, CvScalar value );
void cvSetReal1D( CvArr* arr, int idx1, double value );
void cvSetReal2D( CvArr* arr, int idx1, int idx2, double value );
void cvSetReal3D( CvArr* arr, int idx1,
                            int idx2, int idx3, double value );
void cvSetRealND( CvArr* arr, int* idx, double value );
void cvClearND( CvArr* arr, int* idx );
CvMat* cvGetMat( const CvArr* src, CvMat* header,
                           int* coi ,
                           int allowND );
IplImage* cvGetImage( const CvArr* arr, IplImage* img );
CvArr* cvReshapeMatND( const CvArr* arr,
                                 int sizeof_header, CvArr* header,
                                 int new_cn, int new_dims, int* new_sizes );
CvMat* cvReshape( const CvArr* arr, CvMat* header,
                            int new_cn, int new_rows  );
void cvRepeat( const CvArr* src, CvArr* dst );
void  cvCreateData( CvArr* arr );
void  cvReleaseData( CvArr* arr );
void  cvSetData( CvArr* arr, void* data, int step );
void cvGetRawData( const CvArr* arr, uchar** data,
                             int* step ,
                             CvSize* roi_size );
CvSize cvGetSize( const CvArr* arr );
void  cvCopy( const CvArr* src, CvArr* dst,
                         const CvArr* mask  );
void  cvSet( CvArr* arr, CvScalar scalar,
                        const CvArr* mask  );
void  cvSetZero( CvArr* mat );
void  cvCvtPixToPlane( const void *src, void *dst0, void *dst1,
                                  void *dst2, void *dst3 );
void  cvCvtPlaneToPix( const void *src0, const void *src1,
                                  const void *src2, const void *src3,
                                  void *dst );
void  cvConvertScale( const CvArr *src, CvArr *dst,
                                 double scale ,
                                 double shift  );
void  cvConvertScaleAbs( const void *src, void *dst,
                                    double scale ,
                                    double shift  );
CvRect  cvMaxRect( const CvRect* rect1, const CvRect* rect2 );
void cvBoxPoints( CvBox2D box, CvPoint2D32f pt[4] );
void  cvAdd( const CvArr* srcA, const CvArr* srcB, CvArr* dst,
                        const CvArr* mask );
void  cvAddS( const CvArr* src, CvScalar value, CvArr* dst,
                         const CvArr* mask );
void  cvSub( const CvArr* srcA, const CvArr* srcB, CvArr* dst,
                        const CvArr* mask );
void  cvSubS( const CvArr* src, CvScalar value, CvArr* dst,
                         const CvArr* mask );
void  cvSubRS( const CvArr* src, CvScalar value, CvArr* dst,
                          const CvArr* mask );
void  cvMul( const CvArr* srcA, const CvArr* srcB,
                        CvArr* dst, double scale  );
void  cvDiv( const CvArr* srcA, const CvArr* srcB,
                        CvArr* dst, double scale );
void  cvScaleAdd( const CvArr* srcA, CvScalar scale,
                             const CvArr* srcB, CvArr* dst );
void  cvAddWeighted( const CvArr* srcA, double alpha,
                                const CvArr* srcB, double beta,
                                double gamma, CvArr* dst );
double  cvDotProduct( const CvArr* srcA, const CvArr* srcB );
void cvAnd( const CvArr* src1, const CvArr* src2,
                      CvArr* dst, const CvArr* mask );
void cvAndS( const CvArr* src, CvScalar value,
                       CvArr* dst, const CvArr* mask );
void cvOr( const CvArr* src1, const CvArr* src2,
                     CvArr* dst, const CvArr* mask );
void cvOrS( const CvArr* src, CvScalar value,
                      CvArr* dst, const CvArr* mask );
void cvXor( const CvArr* src1, const CvArr* src2,
                      CvArr* dst, const CvArr* mask );
void cvXorS( const CvArr* src, CvScalar value,
                       CvArr* dst, const CvArr* mask );
void cvNot( const CvArr* src, CvArr* dst );
void cvInRange( const CvArr* src, const CvArr* lower,
                          const CvArr* upper, CvArr* dst );
void cvInRangeS( const CvArr* src, CvScalar lower,
                           CvScalar upper, CvArr* dst );
void cvCmp( const CvArr* src1, const CvArr* src2, CvArr* dst, int cmpOp );
void cvCmpS( const CvArr* src1, double scalar, CvArr* dst, int cmpOp );
void cvMin( const CvArr* src1, const CvArr* src2, CvArr* dst );
void cvMax( const CvArr* src1, const CvArr* src2, CvArr* dst );
void cvMinS( const CvArr* src, double scalar, CvArr* dst );
void cvMaxS( const CvArr* src, double scalar, CvArr* dst );
void  cvCartToPolar( const CvArr* x, const CvArr* y,
                                CvArr* magnitude, CvArr* angle ,
                                int angle_in_degrees );
void  cvPolarToCart( const CvArr* magnitude, const CvArr* angle,
                                CvArr* x, CvArr* y,
                                int angle_in_degrees );
void  cvPow( const CvArr* src, CvArr* dst, double power );
void  cvExp( const CvArr* src, CvArr* dst );
void  cvLog( const CvArr* src, CvArr* dst );
int  cvCheckArr( const CvArr* arr, int flags ,
                            double minVal , double maxVal );
typedef struct CvRandState
{
    int       disttype;
    CvScalar  param[2];
}
CvRandState;
void  cvRandInit( CvRandState* state, double param1,
                             double param2, int seed,
                             int disttype );
void  cvRandSetRange( CvRandState* state, double param1, double param2,
                                 int index );
void  cvRand( CvRandState* state, CvArr* arr );
unsigned  cvRandNext( CvRandState* state );
void  cvCrossProduct( const CvArr* srcA, const CvArr* srcB, CvArr* dst );
void  cvMatMulAdd( const CvArr* srcA, const CvArr* srcB,
                              const CvArr* srcC, CvArr* dst );
void  cvGEMM( const CvArr* srcA, const CvArr* srcB, double alpha,
                         const CvArr* srcC, double beta, CvArr* dst,
                         int tABC );
void  cvMatMulAddS( const CvArr* src, CvArr* dst,
                               const CvMat* transform,
                               const CvMat* shiftvec );
void cvMulTransposed( const CvArr* srcarr,
                                CvArr* dstarr, int order );
void  cvTranspose( const CvArr* src, CvArr* dst );
void  cvFlip( const CvArr* src, CvArr* dst ,
                         int flip_mode );
void   cvSVD( CvArr* A, CvArr* W ,
                         CvArr* U ,
                         CvArr* V ,
                         int flags );
void   cvSVBkSb( const CvArr* warr, const CvArr* uarr,
                            const CvArr* varr, const CvArr* barr,
                            CvArr* xarr, int flags );
double  cvInvert( const CvArr* src, CvArr* dst,
                             int method );
int  cvSolve( const CvArr* A, const CvArr* b, CvArr* x,
                         int method );
double cvDet( const CvArr* mat );
CvScalar cvTrace( const CvArr* mat );
void  cvEigenVV( CvArr* src, CvArr* evects, CvArr* evals, double eps );
void  cvSetIdentity( CvArr* mat, CvScalar value );
void  cvPerspectiveTransform( const CvArr* src, CvArr* dst, const CvArr* mat );
void  cvCalcCovarMatrix( const CvArr** vects, CvArr* covarMatrix, CvArr* avg );
double  cvMahalanobis( const CvArr* srcA, const CvArr* srcB, CvArr* mat );
CvScalar  cvSum( const CvArr* arr );
int  cvCountNonZero( const CvArr* arr );
CvScalar  cvAvg( const CvArr* arr, const CvArr* mask  );
void  cvAvgSdv( const CvArr* arr, CvScalar* mean, CvScalar* std_dev,
                           const CvArr* mask  );
void  cvMinMaxLoc( const CvArr* arr, double* min_val, double* max_val,
                              CvPoint* min_loc ,
                              CvPoint* max_loc ,
                              const CvArr* mask  );
double  cvNorm( const CvArr* arrA, const CvArr* arrB ,
                           int normType,
                           const CvArr* mask  );
void  cvDFT( const CvArr* src, CvArr* dst, int flags );
void  cvMulCcs( const CvArr* srcA, const CvArr* srcB, CvArr* dst );
void  cvDCT( const CvArr* src, CvArr* dst, int flags );
CvMemStorage*  cvCreateMemStorage( int block_size );
CvMemStorage*  cvCreateChildMemStorage( CvMemStorage* parent );
void  cvReleaseMemStorage( CvMemStorage** storage );
void  cvClearMemStorage( CvMemStorage* storage );
void  cvSaveMemStoragePos( const CvMemStorage* storage, CvMemStoragePos* pos );
void  cvRestoreMemStoragePos( CvMemStorage* storage, CvMemStoragePos* pos );
void* cvMemStorageAlloc( CvMemStorage* storage, int size );
CvSeq*  cvCreateSeq( int seq_flags, int header_size,
                             int elem_size, CvMemStorage* storage );
void  cvSetSeqBlockSize( CvSeq* seq, int delta_elements );
char*  cvSeqPush( CvSeq* seq, void* element );
char*  cvSeqPushFront( CvSeq* seq, void* element );
void  cvSeqPop( CvSeq* seq, void* element );
void  cvSeqPopFront( CvSeq* seq, void* element );
void  cvSeqPushMulti( CvSeq* seq, void* elements,
                                 int count, int in_front  );
void  cvSeqPopMulti( CvSeq* seq, void* elements,
                                int count, int in_front  );
char*  cvSeqInsert( CvSeq* seq, int before_index,
                               void* element );
void  cvSeqRemove( CvSeq* seq, int index );
void  cvClearSeq( CvSeq* seq );
char*  cvGetSeqElem( CvSeq* seq, int index, CvSeqBlock** block  );
int  cvSeqElemIdx( const CvSeq* seq, const void* element,
                             CvSeqBlock** block  );
void  cvStartAppendToSeq( CvSeq* seq, CvSeqWriter* writer );
void  cvStartWriteSeq( int seq_flags, int header_size,
                                  int elem_size, CvMemStorage* storage,
                                  CvSeqWriter* writer );
CvSeq*  cvEndWriteSeq( CvSeqWriter* writer );
void   cvFlushSeqWriter( CvSeqWriter* writer );
void cvStartReadSeq( const CvSeq* seq, CvSeqReader* reader,
                               int reverse  );
int  cvGetSeqReaderPos( CvSeqReader* reader );
void   cvSetSeqReaderPos( CvSeqReader* reader, int index,
                                     int is_relative );
void*  cvCvtSeqToArray( CvSeq* seq, CvArr* arr,
                                   CvSlice slice );
CvSeq* cvMakeSeqHeaderForArray( int seq_type, int header_size,
                                           int elem_size, void* elements, int total,
                                           CvSeq* seq, CvSeqBlock* block );
CvSeq* cvSeqSlice( CvSeq* seq, CvSlice slice,
                             CvMemStorage* storage ,
                             int copy_data );
CvSeq* cvCloneSeq( CvSeq* seq, CvMemStorage* storage );
void  cvSeqRemoveSlice( CvSeq* seq, CvSlice slice );
void  cvSeqInsertSlice( CvSeq* seq, int index, const CvArr* from_arr );
typedef int (* CvCmpFunc)
                     (const void* a, const void* b, void* userdata );
void cvSeqSort( CvSeq* seq, CvCmpFunc func, void* userdata );
void cvSeqInvert( CvSeq* seq );
int  cvPartitionSeq( CvSeq* seq, CvMemStorage* storage, CvSeq** comps,
                  CvCmpFunc is_equal, void* userdata, int is_set );
void  cvChangeSeqBlock( CvSeqReader* reader, int direction );
void  cvCreateSeqBlock( CvSeqWriter* writer );
CvSet*  cvCreateSet( int set_flags, int header_size,
                                int elem_size, CvMemStorage* storage );
int  cvSetAdd( CvSet* set_header, CvSetElem* element ,
                          CvSetElem** inserted_element  );
CvSetElem* cvSetNew( CvSet* set_header );
void cvSetRemoveByPtr( CvSet* set_header, void* _elem );
void   cvSetRemove( CvSet* set_header, int index );
CvSetElem*  cvGetSetElem( CvSet* set_header, int index );
void  cvClearSet( CvSet* set_header );
CvGraph*   cvCreateGraph( int graph_flags, int header_size,
                                  int vtx_size, int edge_size,
                                  CvMemStorage* storage );
int  cvGraphAddVtx( CvGraph* graph, CvGraphVtx* vertex ,
                               CvGraphVtx** inserted_vertex  );
void   cvGraphRemoveVtx( CvGraph* graph, int index );
void   cvGraphRemoveVtxByPtr( CvGraph* graph, CvGraphVtx* vtx );
int  cvGraphAddEdge( CvGraph* graph,
                                int start_idx, int end_idx,
                                CvGraphEdge* edge ,
                                CvGraphEdge** inserted_edge  );
int  cvGraphAddEdgeByPtr( CvGraph* graph,
                               CvGraphVtx* start_vtx, CvGraphVtx* end_vtx,
                               CvGraphEdge* edge ,
                               CvGraphEdge** inserted_edge  );
void  cvGraphRemoveEdge( CvGraph* graph, int start_idx, int end_idx );
void  cvGraphRemoveEdgeByPtr( CvGraph* graph, CvGraphVtx* start_vtx,
                                         CvGraphVtx* end_vtx );
CvGraphEdge*  cvFindGraphEdge( CvGraph* graph, int start_idx, int end_idx );
CvGraphEdge*  cvFindGraphEdgeByPtr( CvGraph* graph, CvGraphVtx* start_vtx,
                                               CvGraphVtx* end_vtx );
void  cvClearGraph( CvGraph* graph );
int  cvGraphVtxDegree( CvGraph* graph, int vtx_idx );
int  cvGraphVtxDegreeByPtr( CvGraph* graph, CvGraphVtx* vtx );
typedef struct CvGraphScanner
{
    CvGraphVtx* vtx;
    CvGraphVtx* dst;
    CvGraphEdge* edge;
    CvGraph* graph;
    CvSeq*   stack;
    int      index;
    int      mask;
}
CvGraphScanner;
void  cvStartScanGraph( CvGraph* graph, CvGraphScanner* scanner,
                                  CvGraphVtx* vtx ,
                                  int mask );
void  cvEndScanGraph( CvGraphScanner* scanner );
int  cvNextGraphItem( CvGraphScanner* scanner );
CvGraph* cvCloneGraph( const CvGraph* graph, CvMemStorage* storage );
void cvLUT( const CvArr* srcarr, CvArr* dstarr, const CvArr* lutarr );
void cvSmooth( const CvArr* srcarr, CvArr* dstarr,
                          int smoothtype,
                          int param1 ,
                          int param2 );
void cvIntegral( const CvArr* image, CvArr* sumImage,
                           CvArr* sumSqImage ,
                           CvArr* tiltedSumImage );
void  cvPyrDown( const CvArr* src, CvArr* dst,
                            int filter );
void  cvPyrUp( const CvArr* src, CvArr* dst,
                          int filter );
void cvPyrSegmentation( IplImage* src,
                               IplImage* dst,
                               CvMemStorage *storage,
                               CvSeq **comp,
                               int level, double threshold1,
                               double threshold2 );
void cvSobel( const CvArr* src, CvArr* dst,
                        int xorder, int yorder,
                        int apertureSize );
void cvLaplace( const CvArr* src, CvArr* dst,
                          int apertureSize  );
void  cvCvtColor( const CvArr* src, CvArr* dst, int colorCvtCode );
void  cvResize( const CvArr* src, CvArr* dst,
                           int method );
IplConvKernel*  cvCreateStructuringElementEx(
            int  cols, int  rows, int  anchorX, int  anchorY,
            int shape, int* values  );
void  cvReleaseStructuringElement( IplConvKernel** element );
void  cvErode( const CvArr* src, CvArr* dst,
                          IplConvKernel* element ,
                          int iterations  );
void  cvDilate( const CvArr* src, CvArr* dst,
                           IplConvKernel* element ,
                           int iterations  );
void  cvMorphologyEx( const CvArr* src, CvArr* dst,
                                 CvArr* temp, IplConvKernel* element,
                                 int operation, int iterations  );
typedef struct CvMoments
{
    double  m00, m10, m01, m20, m11, m02, m30, m21, m12, m03;
    double  mu20, mu11, mu02, mu30, mu21, mu12, mu03;
    double  inv_sqrt_m00;
} CvMoments;
void cvMoments( const CvArr* arr, CvMoments* moments, int binary );
double  cvGetSpatialMoment( CvMoments* moments, int x_order, int y_order );
double  cvGetCentralMoment( CvMoments* moments, int x_order, int y_order );
double  cvGetNormalizedCentralMoment( CvMoments* moments,
                                                 int x_order, int y_order );
typedef struct CvHuMoments
{
    double hu1, hu2, hu3, hu4, hu5, hu6, hu7;
} CvHuMoments;
void cvGetHuMoments( CvMoments*  moments, CvHuMoments*  hu_moments );
void  cvLine( CvArr* arr, CvPoint pt1, CvPoint pt2,
                         double color, int thickness ,
                         int connectivity );
void  cvLineAA( CvArr* arr, CvPoint pt1, CvPoint pt2,
                           double color, int scale );
void  cvRectangle( CvArr* arr, CvPoint pt1, CvPoint pt2,
                              double color, int thickness );
void  cvCircle( CvArr* arr, CvPoint center, int radius,
                           double color, int thickness );
void  cvCircleAA( CvArr* arr, CvPoint center, int radius,
                             double color, int scale  );
void  cvEllipse( CvArr* arr, CvPoint center, CvSize axes,
                            double angle, double startAngle, double endAngle,
                            double color, int thickness );
void  cvEllipseBox( CvArr* arr, CvBox2D box,
                               double color, int thickness );
void  cvEllipseAA( CvArr* arr, CvPoint center, CvSize axes,
                              double angle, double startAngle,
                              double endAngle, double color,
                              int scale  );
void  cvFillConvexPoly( CvArr* arr, CvPoint* pts, int npts, double color );
void  cvFillPoly( CvArr* arr, CvPoint** pts,
                             int* npts, int contours, double color );
void  cvPolyLine( CvArr* arr, CvPoint** pts, int* npts, int contours,
                             int closed, double color,
                             int thickness ,
                             int connectivity );
void  cvPolyLineAA( CvArr* arr, CvPoint** pts, int* npts, int contours,
                               int closed, double color, int scale  );
typedef struct CvFont
{
    const int*  data;
    CvSize      size;
    int         italic_scale;
    int         thickness;
    int         dx;
} CvFont;
void  cvInitFont( CvFont* font, int font_face,
                             double hscale, double vscale,
                             double italic_scale ,
                             int thickness  );
void  cvPutText( CvArr* arr, const char* text, CvPoint org,
                            CvFont* font, double color );
void  cvGetTextSize( const char* text_string, CvFont* font,
                                CvSize* text_size, int* ymin );
typedef struct CvLineIterator
{
    uchar* ptr;
    int  err;
    int  plus_delta;
    int  minus_delta;
    int  plus_step;
    int  minus_step;
} CvLineIterator;
int  cvInitLineIterator( const CvArr* arr, CvPoint pt1, CvPoint pt2,
                                    CvLineIterator* lineIterator,
                                    int connectivity );
int  cvSampleLine( const CvArr* arr, CvPoint pt1, CvPoint pt2, void* buffer,
                              int connectivity );
void  cvGetRectSubPix( const CvArr* src, CvArr* dst, CvPoint2D32f center );
void  cvGetQuadrangleSubPix( const CvArr* src, CvArr* dstarr,
                                        const CvArr* matrixarr,
                                        int fillOutliers ,
                                        CvScalar fillvalue );
void  cvMatchTemplate( const CvArr* arr, const CvArr* templ,
                                  CvArr* result, int method );
typedef float (* CvDistanceFunction)
                     ( const float* a, const float* b, void* user_param );
float  cvCalcEMD2( const CvArr* signature1,
                              const CvArr* signature2,
                              CvDisType dist_type,
                              CvDistanceFunction dist_func ,
                              const CvArr* cost_matrix ,
                              CvArr* flow ,
                              float* lower_bound ,
                              void* user_param );
typedef struct _CvContourScanner* CvContourScanner;
typedef enum CvContourRetrievalMode
{
    CV_RETR_EXTERNAL = 0,
    CV_RETR_LIST     = 1,
    CV_RETR_CCOMP    = 2,
    CV_RETR_TREE     = 3
}
CvContourRetrievalMode;
typedef enum CvChainApproxMethod
{
    CV_CHAIN_CODE             = 0,
    CV_CHAIN_APPROX_NONE      = 1,
    CV_CHAIN_APPROX_SIMPLE    = 2,
    CV_CHAIN_APPROX_TC89_L1   = 3,
    CV_CHAIN_APPROX_TC89_KCOS = 4,
    CV_LINK_RUNS              = 5
} CvChainApproxMethod;
int  cvFindContours( CvArr* arr, CvMemStorage* storage,
                           CvSeq**  firstContour,
                           int  headerSize,
                           CvContourRetrievalMode mode,
                           CvChainApproxMethod method );
CvContourScanner   cvStartFindContours( CvArr* arr, CvMemStorage* storage,
                                        int header_size, CvContourRetrievalMode mode,
                                        CvChainApproxMethod method );
CvSeq*  cvFindNextContour( CvContourScanner scanner );
void   cvSubstituteContour( CvContourScanner scanner, CvSeq* newContour );
CvSeq*  cvEndFindContours( CvContourScanner* scanner );
void  cvDrawContours( CvArr *img, CvSeq* contour,
                                double external_color, double hole_color,
                                int max_level, int thickness ,
                                int connectivity );
typedef struct CvTreeNodeIterator
{
    const void* node;
    int level;
    int maxLevel;
}
CvTreeNodeIterator;
void cvInitTreeNodeIterator( CvTreeNodeIterator* treeIterator,
                                   const void* first, int maxLevel );
void* cvNextTreeNode( CvTreeNodeIterator* treeIterator );
void* cvPrevTreeNode( CvTreeNodeIterator* treeIterator );
void cvInsertNodeIntoTree( void* node, void* parent, void* frame );
void cvRemoveNodeFromTree( void* node, void* frame );
CvSeq* cvTreeToNodeSeq( const void* first, int header_size,
                                  CvMemStorage* storage );
CvSeq* cvApproxChains( CvSeq* src_seq, CvMemStorage* storage,
                            CvChainApproxMethod method ,
                            double parameter ,
                            int  minimal_perimeter ,
                            int  recursive );
typedef struct CvChainPtReader
{
/*  CV_SEQ_READER_FIELDS() */
    int          header_size;
    CvSeq*       seq;
    CvSeqBlock*  block;
    char*        ptr;
    char*        block_min;
    char*        block_max;
    int          delta_index;
    char*        prev_elem;
    char      code;
    CvPoint  pt;
    char      deltas[8][2];
    int       reserved[2];
} CvChainPtReader;
void  cvStartReadChainPoints( CvChain* chain, CvChainPtReader* reader );
CvPoint   cvReadChainPoint( CvChainPtReader* reader );
void  cvAbsDiff( const CvArr* srcA, const CvArr* srcB, CvArr* dst );
void  cvAbsDiffS( const CvArr* src, CvArr* dst, CvScalar value );
void  cvCalcOpticalFlowLK( const CvArr* srcA, const CvArr* srcB,
                                      CvSize winSize, CvArr* velx, CvArr* vely );
void  cvCalcOpticalFlowBM( const CvArr* srcA, const CvArr* srcB,
                                      CvSize blockSize, CvSize shiftSize,
                                      CvSize maxRange, int usePrevious,
                                      CvArr* velx, CvArr* vely );
void  cvCalcOpticalFlowHS( const CvArr* srcA, const CvArr* srcB,
                                      int usePrevious, CvArr* velx, CvArr* vely,
                                      double _lambda, CvTermCriteria criteria );
void  cvCalcOpticalFlowPyrLK( const CvArr*  imgA, const CvArr*  imgB,
                                         CvArr*  pyrA, CvArr*  pyrB,
                                         CvPoint2D32f* featuresA,
                                         CvPoint2D32f* featuresB,
                                         int       count,
                                         CvSize    winSize,
                                         int       level,
                                         char*     status,
                                         float*    error,
                                         CvTermCriteria criteria,
                                         int       flags );
void  cvCalcAffineFlowPyrLK( const CvArr*  imgA, const CvArr*  imgB,
                                        CvArr*  pyrA, CvArr*  pyrB,
                                        CvPoint2D32f* featuresA,
                                        CvPoint2D32f* featuresB,
                                        float*  matrices, int  count,
                                        CvSize  winSize, int  level,
                                        char*  status, float* error,
                                        CvTermCriteria criteria, int flags );
void    cvUpdateMotionHistory( const CvArr* silhouette, CvArr* mhi,
                                          double timestamp, double mhiDuration );
void    cvCalcMotionGradient( const CvArr* mhi, CvArr* mask, CvArr* orientation,
                                         double delta1, double delta2,
                                         int aperture_size );
double  cvCalcGlobalOrientation( const CvArr* orientation, const CvArr* mask,
                                            const CvArr* mhi, double curr_mhi_timestamp,
                                            double mhi_duration );
CvSeq*  cvSegmentMotion( const CvArr* mhi, CvArr* seg_mask,
                                    CvMemStorage* storage,
                                    double timestamp, double seg_thresh );
void  cvAcc( const CvArr* image, CvArr* sum,
                        const CvArr* mask  );
void  cvSquareAcc( const CvArr* image, CvArr* sqSum,
                              const CvArr* mask  );
void  cvMultiplyAcc( const CvArr* imgA, const CvArr* imgB, CvArr* acc,
                                const CvArr* mask  );
void  cvRunningAvg( const CvArr* imgY, CvArr* imgU, double alpha,
                               const CvArr* mask  );
int  cvCamShift( const CvArr* imgProb, CvRect  windowIn,
                           CvTermCriteria criteria, CvConnectedComp* out,
                           CvBox2D* box );
int  cvMeanShift( const CvArr* imgProb, CvRect  windowIn,
                            CvTermCriteria criteria, CvConnectedComp* out );
typedef struct CvConDensation
{
    int MP;
    int DP;
    float* DynamMatr;
    float* State;
    int SamplesNum;
    float** flSamples;
    float** flNewSamples;
    float* flConfidence;
    float* flCumulative;
    float* Temp;
    float* RandomSample;
    CvRandState* RandS;
} CvConDensation;
CvConDensation*  cvCreateConDensation( int DP, int MP, int SamplesNum);
void  cvReleaseConDensation( CvConDensation** ConDensation);
void  cvConDensUpdateByTime( CvConDensation* ConDens);
void  cvConDensInitSampleSet( CvConDensation* conDens, CvMat* lowerBound,CvMat* upperBound);
typedef struct CvKalman
{
    int MP;
    int DP;
    int CP;
    float* PosterState;
    float* PriorState;
    float* DynamMatr;
    float* MeasurementMatr;
    float* MNCovariance;
    float* PNCovariance;
    float* KalmGainMatr;
    float* PriorErrorCovariance;
    float* PosterErrorCovariance;
    float* Temp1;
    float* Temp2;
    CvMat* state_pre;
    CvMat* state_post;
    CvMat* transition_matrix;
    CvMat* control_matrix;
    CvMat* measurement_matrix;
    CvMat* process_noise_cov;
    CvMat* measurement_noise_cov;
    CvMat* error_cov_pre;
    CvMat* gain;
    CvMat* error_cov_post;
    CvMat* temp1;
    CvMat* temp2;
    CvMat* temp3;
    CvMat* temp4;
    CvMat* temp5;
}
CvKalman;
CvKalman* cvCreateKalman( int dynamParams, int measureParams,
                                    int controlParams );
void  cvReleaseKalman( CvKalman** Kalman);
const CvMat*  cvKalmanPredict( CvKalman* Kalman,
                                         const CvMat* control );
const CvMat*  cvKalmanCorrect( CvKalman* Kalman, const CvMat* measurement );
typedef long CvSubdiv2DEdge;
typedef struct CvQuadEdge2D
{
/*  CV_QUADEDGE2D_FIELDS() */
    int flags;
    struct CvSubdiv2DPoint* pt[4];
    CvSubdiv2DEdge  next[4];
}
CvQuadEdge2D;
typedef struct CvSubdiv2DPoint
{
/*  CV_SUBDIV2D_POINT_FIELDS() */
    int            flags;
    CvSubdiv2DEdge first;
    CvPoint2D32f   pt;
}
CvSubdiv2DPoint;
typedef struct CvSubdiv2D
{
/*  CV_SUBDIV2D_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvSetElem* free_elems;
    CvSet* edges;
    int  quad_edges;
    int  is_geometry_valid;
    CvSubdiv2DEdge recent_edge;
    CvPoint2D32f  topleft;
    CvPoint2D32f  bottomright;
}
CvSubdiv2D;
typedef enum CvSubdiv2DPointLocation
{
    CV_PTLOC_ERROR = -2,
    CV_PTLOC_OUTSIDE_RECT = -1,
    CV_PTLOC_INSIDE = 0,
    CV_PTLOC_VERTEX = 1,
    CV_PTLOC_ON_EDGE = 2
}
CvSubdiv2DPointLocation;
typedef enum CvNextEdgeType
{
    CV_NEXT_AROUND_ORG   = 0x00,
    CV_NEXT_AROUND_DST   = 0x22,
    CV_PREV_AROUND_ORG   = 0x11,
    CV_PREV_AROUND_DST   = 0x33,
    CV_NEXT_AROUND_LEFT  = 0x13,
    CV_NEXT_AROUND_RIGHT = 0x31,
    CV_PREV_AROUND_LEFT  = 0x20,
    CV_PREV_AROUND_RIGHT = 0x02
}
CvNextEdgeType;
void  cvInitSubdivDelaunay2D( CvSubdiv2D* subdiv, CvRect rect );
CvSubdiv2D*  cvCreateSubdiv2D( int subdiv_type, int header_size,
                                       int vtx_size, int quadedge_size,
                                       CvMemStorage* storage );
CvSubdiv2D* cvCreateSubdivDelaunay2D( CvRect rect, CvMemStorage* storage );
CvSubdiv2DPoint*  cvSubdivDelaunay2DInsert( CvSubdiv2D* subdiv, CvPoint2D32f pt);
CvSubdiv2DPointLocation  cvSubdiv2DLocate(
                               CvSubdiv2D* subdiv, CvPoint2D32f pt,
                               CvSubdiv2DEdge *_edge,
                               CvSubdiv2DPoint** _point  );
void  cvCalcSubdivVoronoi2D( CvSubdiv2D* subdiv );
void  cvClearSubdivVoronoi2D( CvSubdiv2D* subdiv );
CvSubdiv2DPoint* cvFindNearestPoint2D( CvSubdiv2D* subdiv, CvPoint2D32f pt );
CvSubdiv2DEdge  cvSubdiv2DNextEdge( CvSubdiv2DEdge edge );
CvSubdiv2DEdge  cvSubdiv2DRotateEdge( CvSubdiv2DEdge edge, int rotate );
CvSubdiv2DEdge  cvSubdiv2DSymEdge( CvSubdiv2DEdge edge );
CvSubdiv2DEdge  cvSubdiv2DGetEdge( CvSubdiv2DEdge edge, CvNextEdgeType type );
CvSubdiv2DPoint*  cvSubdiv2DEdgeOrg( CvSubdiv2DEdge edge );
CvSubdiv2DPoint*  cvSubdiv2DEdgeDst( CvSubdiv2DEdge edge );
double  cvTriangleArea( CvPoint2D32f a, CvPoint2D32f b, CvPoint2D32f c );
CvSeq*  cvApproxPoly( const void* src_seq,
                                 int header_size, CvMemStorage* storage,
                                 int method, double parameter,
                                 int parameter2 );
double  cvArcLength( const void* curve,
                                CvSlice slice ,
                                int is_closed );
CvRect  cvBoundingRect( const void* points, int update  );
double  cvContourArea( const void* contour,
                                  CvSlice slice );
CvBox2D  cvMinAreaRect2( const void* points,
                                    CvMemStorage* storage );
void  cvMinEnclosingCircle( const void* points,
                                       CvPoint2D32f* center, float* radius );
double  cvMatchShapes( const void* contour1, const void* contour2,
                                  int method, double parameter );
typedef struct CvContourTree
{
/*  CV_SEQUENCE_FIELDS() */
    int       flags;
    int       header_size;
    struct    CvSeq* h_prev;
    struct    CvSeq* h_next;
    struct    CvSeq* v_prev;
    struct    CvSeq* v_next;
    int       total;
    int       elem_size;
    char*     block_max;
    char*     ptr;
    int       delta_elems;
    CvMemStorage* storage;
    CvSeqBlock* free_blocks;
    CvSeqBlock* first;
    CvPoint p1;
    CvPoint p2;
} CvContourTree;
CvContourTree*   cvCreateContourTree( CvSeq* contour, CvMemStorage* storage,
                                                 double threshold );
CvSeq*  cvContourFromContourTree( CvContourTree *tree,
                                          CvMemStorage* storage,
                                          CvTermCriteria criteria );
double  cvMatchContourTrees( CvContourTree *tree1,
                                        CvContourTree *tree2,
                                        int method, double threshold );
void  cvCalcPGH( const CvSeq* contour, CvHistogram* hist );
CvSeq* cvConvexHull2( const CvArr* input,
                                void* hull_storage ,
                                int orientation,
                                int return_points );
int  cvCheckContourConvexity( const CvArr* contour );
typedef struct CvConvexityDefect
{
    CvPoint* start;
    CvPoint* end;
    CvPoint* depth_point;
    float depth;
} CvConvexityDefect;
CvSeq*  cvConvexityDefects( const CvArr* contour, const CvArr* convexhull,
                                      CvMemStorage* storage );
CvBox2D cvFitEllipse2( const CvArr* points );
CvHistogram*  cvCreateHist( int dims, int* sizes, int type,
                                       float** ranges ,
                                       int uniform );
void  cvSetHistBinRanges( CvHistogram* hist, float** ranges,
                                    int uniform );
CvHistogram*  cvMakeHistHeaderForArray(
                            int  dims, int* sizes, CvHistogram* hist,
                            float* data, float** ranges ,
                            int uniform );
void  cvReleaseHist( CvHistogram** hist );
void  cvClearHist( CvHistogram* hist );
void  cvGetMinMaxHistValue( const CvHistogram* hist,
                                    float* value_min, float* value_max,
                                    int* idx_min ,
                                    int* idx_max );
void  cvNormalizeHist( CvHistogram* hist, double factor );
void  cvThreshHist( CvHistogram* hist, double thresh );
double  cvCompareHist( const CvHistogram* hist1,
                                  const CvHistogram* hist2,
                                  int method);
void  cvCopyHist( const CvHistogram* src, CvHistogram** dst );
void  cvCalcBayesianProb( CvHistogram** src, int number,
                                    CvHistogram** dst);
void  cvCalcArrHist( CvArr** arr, CvHistogram* hist,
                                int doNotClear ,
                                const CvArr* mask  );
void  cvCalcHist( IplImage** img, CvHistogram* hist,
                             int doNotClear ,
                             const CvArr* mask  );
void  cvCalcArrBackProject( CvArr** img, CvArr* dst,
                                       const CvHistogram* hist );
void  cvCalcArrBackProjectPatch( CvArr** img, CvArr* dst, CvSize range,
                                            CvHistogram* hist, int method,
                                            double normFactor );
void  cvCalcProbDensity( const CvHistogram* hist, const CvHistogram* hist_mask,
                                    CvHistogram* hist_dens, double scale );
void  cvSnakeImage( const IplImage* src, CvPoint* points,
                            int  length, float* alpha,
                            float* beta, float* gamma,
                            int coeffUsage, CvSize  win,
                            CvTermCriteria criteria, int calcGradient );
void  cvCalcImageHomography(float *line, CvPoint3D32f* center,
                                     float* intrinsic, float* homography);
void  cvDistTransform( const CvArr* src, CvArr* dst,
                                  CvDisType disType,
                                  int maskSize ,
                                  const float* mask );
void  cvThreshold( const CvArr*  src, CvArr*  dst,
                              double  thresh, double  maxValue, int type );
void  cvAdaptiveThreshold( const CvArr* src, CvArr* dst, double maxValue,
                                      int method,
                                      int type,
                                      int blockSize ,
                                      double param1 );
void  cvFloodFill( CvArr* arr, CvPoint seedPoint,
                              double newVal, double loDiff ,
                              double upDiff ,
                              CvConnectedComp* comp ,
                              int flags ,
                              CvArr* mask );
void  cvCanny( const CvArr* src, CvArr* dst, double low_threshold,
                          double high_threshold, int  aperture_size  );
void cvPreCornerDetect( const CvArr* src, CvArr* dst,
                                  int aperture_size  );
void  cvCornerEigenValsAndVecs( const CvArr* src, CvArr* eigenvv,
                                          int blockSize,
                                          int aperture_size  );
void  cvCornerMinEigenVal( const CvArr* src, CvArr* eigenval,
                                     int blockSize, int aperture_size  );
void  cvFindCornerSubPix( const CvArr* src,CvPoint2D32f*  corners,
                                     int count, CvSize win,CvSize zero_zone,
                                     CvTermCriteria  criteria );
void  cvGoodFeaturesToTrack( const CvArr* image, CvArr* eig_image,
                                       CvArr* temp_image, CvPoint2D32f* corners,
                                       int* corner_count, double  quality_level,
                                       double  min_distance,
                                       const CvArr* mask );
CvSeq*  cvHoughLines2( CvArr* image, void* line_storage, int method,
                                  double rho, double theta, int threshold,
                                  double param1 , double param2 );
void  cvProject3D( CvPoint3D32f* points3D, int count,
                              CvPoint2D32f* points2D, int xIndx, int yIndx );
void  cvFitLine( const CvArr* points, CvDisType dist, double param,
                            double reps, double aeps, float* line );
void  cvCalcCovarMatrixEx( int nObjects, void* input, int ioFlags,
                                      int ioBufSize, uchar* buffer, void* userData,
                                      IplImage* avg, float* covarMatrix );
void  cvCalcEigenObjects( int nObjects, void* input, void* output,
                                    int ioFlags, int ioBufSize, void* userData,
                                    CvTermCriteria* calcLimit, IplImage* avg,
                                    float* eigVals );
double  cvCalcDecompCoeff( IplImage* obj, IplImage* eigObj, IplImage* avg );
void  cvEigenDecomposite( IplImage* obj, int nEigObjs, void* eigInput,
                                    int ioFlags, void* userData, IplImage* avg,
                                    float* coeffs );
void  cvEigenProjection( void* eigInput, int nEigObjs, int ioFlags,
                                   void* userData, float* coeffs, IplImage* avg,
                                   IplImage* proj );
typedef struct CvEHMMState
{
    int num_mix;
    float* mu;
    float* inv_var;
    float* log_var_val;
    float* weight;
} CvEHMMState;
typedef union {
    CvEHMMState* state;
    struct CvEHMM* ehmm;
} CvEHMM_u;
typedef struct CvEHMM
{
    int level;
    int num_states;
    float*  transP;
    float** obsProb;
    CvEHMM_u u;
} CvEHMM;
CvEHMM*  cvCreate2DHMM( int* stateNumber, int* numMix, int obsSize );
void  cvRelease2DHMM( CvEHMM** hmm );
typedef struct CvImgObsInfo
{
    int obs_x;
    int obs_y;
    int obs_size;
    float* obs;
    int* state;
    int* mix;
} CvImgObsInfo;
CvImgObsInfo*  cvCreateObsInfo( CvSize numObs, int obsSize );
void  cvReleaseObsInfo( CvImgObsInfo** obs_info );
void  cvImgToObs_DCT( const CvArr* arr, float* obs, CvSize dctSize,
                                 CvSize obsSize, CvSize delta );
void  cvUniformImgSegm( CvImgObsInfo* obs_info, CvEHMM* ehmm );
void  cvInitMixSegm( CvImgObsInfo** obs_info_array,
                               int num_img, CvEHMM* hmm );
void  cvEstimateHMMStateParams( CvImgObsInfo** obs_info_array,
                                        int num_img, CvEHMM* hmm );
void  cvEstimateTransProb( CvImgObsInfo** obs_info_array,
                                   int num_img, CvEHMM* hmm );
void  cvEstimateObsProb( CvImgObsInfo* obs_info,
                                   CvEHMM* hmm );
float  cvEViterbi( CvImgObsInfo* obs_info, CvEHMM* hmm );
void  cvMixSegmL2( CvImgObsInfo** obs_info_array,
                             int num_img, CvEHMM* hmm );
void  cvKMeans2( const CvArr* samples, int cluster_count,
                            CvArr* cluster_idx, CvTermCriteria termcrit );
void  cvUnDistortOnce( const CvArr* srcImage, CvArr* dstImage,
                                  const float* intrMatrix,
                                  const float* distCoeffs,
                                  int interpolate  );
void  cvUnDistortInit( const CvArr* srcImage, CvArr* undistMap,
                                  const float* intrMatrix,
                                  const float* distCoeffs,
                                  int interpolate  );
void  cvUnDistort( const CvArr* srcImage, CvArr* dstImage,
                              const CvArr* undistMap, int interpolate );
void  cvConvertMap( const CvArr* srcImage, const CvArr* flUndistMap,
                               CvArr* undistMap, int iterpolate  );
void  cvCalibrateCamera( int           numImages,
                                    int*          numPoints,
                                    CvSize        imageSize,
                                    CvPoint2D32f* imagePoints32f,
                                    CvPoint3D32f* objectPoints32f,
                                    CvVect32f     distortion32f,
                                    CvMatr32f     cameraMatrix32f,
                                    CvVect32f     transVects32f,
                                    CvMatr32f     rotMatrs32f,
                                    int           useIntrinsicGuess);
void  cvCalibrateCamera_64d( int           numImages,
                                       int*          numPoints,
                                       CvSize        imageSize,
                                       CvPoint2D64d* imagePoints,
                                       CvPoint3D64d* objectPoints,
                                       CvVect64d     distortion,
                                       CvMatr64d     cameraMatrix,
                                       CvVect64d     transVects,
                                       CvMatr64d     rotMatrs,
                                       int           useIntrinsicGuess );
void  cvFindExtrinsicCameraParams( int           numPoints,
                                             CvSize        imageSize,
                                             CvPoint2D32f* imagePoints32f,
                                             CvPoint3D32f* objectPoints32f,
                                             CvVect32f     focalLength32f,
                                             CvPoint2D32f  principalPoint32f,
                                             CvVect32f     distortion32f,
                                             CvVect32f     rotVect32f,
                                             CvVect32f     transVect32f);
void  cvFindExtrinsicCameraParams_64d( int           numPoints,
                                                 CvSize        imageSize,
                                                 CvPoint2D64d* imagePoints,
                                                 CvPoint3D64d* objectPoints,
                                                 CvVect64d     focalLength,
                                                 CvPoint2D64d  principalPoint,
                                                 CvVect64d     distortion,
                                                 CvVect64d     rotVect,
                                                 CvVect64d     transVect);
void  cvRodrigues( CvMat* rotMatrix, CvMat* rotVector,
                              CvMat* jacobian, int convType);
void  cvProjectPoints( int             numPoints,
                                 CvPoint3D64d*   objectPoints,
                                 CvVect64d       rotVect,
                                 CvVect64d       transVect,
                                 CvVect64d       focalLength,
                                 CvPoint2D64d    principalPoint,
                                 CvVect64d       distortion,
                                 CvPoint2D64d*   imagePoints,
                                 CvVect64d       derivPointsRot,
                                 CvVect64d       derivPointsTrans,
                                 CvVect64d       derivPointsFocal,
                                 CvVect64d       derivPointsPrincipal,
                                 CvVect64d       derivPointsDistort);
void cvProjectPointsSimple(  int numPoints,
                                    CvPoint3D64d * objectPoints,
                                    CvVect64d rotMatr,
                                    CvVect64d transVect,
                                    CvMatr64d cameraMatrix,
                                    CvVect64d distortion,
                                    CvPoint2D64d* imagePoints);
int  cvFindChessBoardCornerGuesses( const CvArr* arr, CvArr* thresh,
                                               CvMemStorage* storage, CvSize etalon_size,
                                               CvPoint2D32f* corners,
                                               int *corner_count );
typedef struct CvPOSITObject CvPOSITObject;
CvPOSITObject*  cvCreatePOSITObject( CvPoint3D32f* points, int numPoints );
void  cvPOSIT(  CvPOSITObject* pObject, CvPoint2D32f* imagePoints,
                           double focalLength, CvTermCriteria criteria,
                           CvMatr32f rotation, CvVect32f translation);
void  cvReleasePOSITObject( CvPOSITObject**  ppObject );
void cvMake2DPoints(CvMat* srcPoint,CvMat* dstPoint);
void cvMake3DPoints(CvMat* srcPoint,CvMat* dstPoint);
int cvSolveCubic(CvMat* coeffs,CvMat* result);
int cvFindFundamentalMat( CvMat* points1, CvMat* points2,
                                     CvMat* fundMatr, int method,
                                     double param1, double param2,
                                     CvMat* status );
void cvComputeCorrespondEpilines(CvMat* points,
                                            int pointImageID,
                                            CvMat* fundMatr,
                                            CvMat* corrLines);
void  cvMakeScanlines( const CvMatrix3* matrix,
                                CvSize     imgSize,
                                int*       scanlines_1,
                                int*       scanlines_2,
                                int*       lens_1,
                                int*       lens_2,
                                int*       numlines);
void  cvPreWarpImage( int       numLines,
                               IplImage* img,
                               uchar*    dst,
                               int*      dst_nums,
                               int*      scanlines);
void  cvFindRuns( int    numLines,
                           uchar* prewarp_1,
                           uchar* prewarp_2,
                           int*   line_lens_1,
                           int*   line_lens_2,
                           int*   runs_1,
                           int*   runs_2,
                           int*   num_runs_1,
                           int*   num_runs_2);
void  cvDynamicCorrespondMulti( int  lines,
                                         int* first,
                                         int* first_runs,
                                         int* second,
                                         int* second_runs,
                                         int* first_corr,
                                         int* second_corr);
void  cvMakeAlphaScanlines( int*  scanlines_1,
                                     int*  scanlines_2,
                                     int*  scanlines_a,
                                     int*  lens,
                                     int   numlines,
                                     float alpha);
void  cvMorphEpilinesMulti( int    lines,
                                     uchar* first_pix,
                                     int*   first_num,
                                     uchar* second_pix,
                                     int*   second_num,
                                     uchar* dst_pix,
                                     int*   dst_num,
                                     float  alpha,
                                     int*   first,
                                     int*   first_runs,
                                     int*   second,
                                     int*   second_runs,
                                     int*   first_corr,
                                     int*   second_corr);
void  cvPostWarpImage( int       numLines,
                                uchar*    src,
                                int*      src_nums,
                                IplImage* img,
                                int*      scanlines);
void  cvDeleteMoire( IplImage*  img);
int  cvLoadPrimitives( const char* proc_type  );
int  cvFillInternalFuncsTable(void* table);
void  cvGetLibraryInfo( const char** version, int* loaded,
                                   const char** loaded_modules );
CVStatus cvGetErrStatus( void );
void cvSetErrStatus( CVStatus status );
int  cvGetErrMode( void );
void cvSetErrMode( int mode );
CVStatus cvError( CVStatus code, const char *func,
                         const char *context, const char *file, int line);
const char* cvErrorStr( CVStatus status );
CVStatus cvNulDevReport( CVStatus status, const char *funcName,
                                const char *context, const char *file, int line );
CVStatus cvStdErrReport( CVStatus status, const char *funcName,
                                const char *context, const char *file, int line );
CVStatus cvGuiBoxReport( CVStatus status, const char *funcName,
                                const char *context, const char *file, int line);
void cvGetCallStack(CvStackRecord** stack, int* size);
void cvStartProfile( const char* call, const char* file, int line );
void cvEndProfile( const char* file, int line );
typedef struct CvFileStorage CvFileStorage;
CvFileStorage*  cvOpenFileStorage( const char* filename,
                                              CvMemStorage* storage,
                                              int flags );
void cvReleaseFileStorage( CvFileStorage** storage );
typedef struct CvAttrList
{
    char** attr;
    struct CvAttrList* next;
}
CvAttrList;
CvAttrList cvAttrList( char** attr ,
                                 CvAttrList* next  );
const char* cvAttrValue( const CvAttrList* attr, const char* attr_name );
struct CvTypeInfo;
typedef struct CvFileNode
{
/*  CV_TREE_NODE_FIELDS(CvFileNode) */
    int       flags;
    int       header_size;
    struct    CvFileNode* h_prev;
    struct    CvFileNode* h_next;
    struct    CvFileNode* v_prev;
    struct    CvFileNode* v_next;
    const char* tagname;
    const char* name;
    CvAttrList* attr;
    struct CvFileNode* hash_next;
    unsigned hash_val;
    int elem_size;
    struct CvTypeInfo* typeinfo;
    const char* body;
    const void* content;
}
CvFileNode;
void cvWrite( CvFileStorage* storage, const char* name,
                         const void* struct_ptr,
                         CvAttrList attributes,
                         int flags );
void cvStartWriteStruct( CvFileStorage* storage, const char* name,
                                    const char* type_name ,
                                    const void* struct_ptr ,
                                    CvAttrList attributes );
void cvEndWriteStruct( CvFileStorage* storage );
void cvWriteElem( CvFileStorage* storage,
                             const char* name,
                             const char* elem_spec,
                             const void* data_ptr );
CvFileNode* cvGetFileNode( CvFileStorage* storage, const char* name );
const void* cvReadFileNode( CvFileStorage* storage, CvFileNode* node,
                                       CvAttrList** attr_list );
const void* cvRead( CvFileStorage* storage, const char* name,
                               CvAttrList** attr_list );
int cvReadElem( CvFileStorage* storage, const char* name, void* data_ptr );
void cvRelease( void** struct_ptr );
void* cvClone( const void* struct_ptr );
