#! /usr/bin/env python

import sys, os, shutil, logging
import tree
from LN import LN
from myexceptions import *

suffixes = ('.c', '.h')

killthese = ('.strip', '.data', '.nest', '.pps', '.tmpsrc', '.src',
            '.src.strip', '.src.data', '.src.nest', '.src.pps',
            '.src.tmpsrc', '.src.src')

message = """\
USAGE: ./docheck <dir_or_file>

    If "dir_or_file" is a directory name, then all the
    files in the directory with extensions ".c" or ".h" will be processed.
    If only a file name is given, the directory defaults to the current
    working directory."""

logger = None

def runtest(filename):
    srcname = filename + '.src'
    srcsrcname = filename + '.src.src'
    diffname = filename + '.diff'

    tree.Run(filename)
    tree.Run(srcname)
    command = 'diff %s %s > %s' % (srcname, srcsrcname, diffname)
    os.system(command)
    if os.stat(diffname).st_size > 0:
        raise LocalError, 'file failed test'

def testfile(filename, topworkdir):
    head, localname = os.path.split(filename)
    localerrdir = 'ERR' + localname + 'ERR'
    workdir = os.path.join(topworkdir, localerrdir)
    os.mkdir(workdir)
    shutil.copy(filename, workdir)
    workname = os.path.join(workdir, localname)
    try:
        runtest(workname)
    except catchtuple, message:
        print >> sys.stderr, \
            'exception raised by "%s". See "%s".' % \
               (localname, os.path.join(topworkdir, 'docheck.log'))
        logger.exception(50*'=' + '\n%s:\n' % workname)
        return False
    else:
        shutil.rmtree(workdir, ignore_errors=True)
        return True

def testbunch(directory, filename, topworkdir):
    if not os.path.isdir(directory):
        raise ParameterError, 'first parameter must be a directory'
    if filename is not None and not os.path.isfile(filename):
        raise ParameterError, 'second parameter must be a file name or None'
    if not os.path.isdir(topworkdir):
        raise ParameterError, 'third parameter must be a directory'
    if filename is None:
        names = os.listdir(directory)
    else:
        names = [filename]
    newnames = []
    for name in names:
        relname = os.path.normpath(os.path.join(directory, name))
        root, ext = os.path.splitext(relname)
        if ext in suffixes and os.path.isfile(relname):
            newnames.append(relname)
    newnames.sort()
    for name in newnames:
        print name
        testfile(name, topworkdir)

def filestuff():
    global logger

    for name in sys.argv[1:]:
        if not os.path.isdir(name) and not os.path.isfile(name):
            sys.exit(message)
    name = sys.argv[1]
    isadir = os.path.isdir(name)
    isafile = os.path.isfile(name)
    if isadir:
        directory = name
        filename = None
    else:
        directory, filename = os.path.split(name)
        if directory == '':
            directory = '.'

    topworkdir = 'ERRdocheckERR'
    if os.path.isdir(topworkdir):
        shutil.rmtree(topworkdir)
    os.mkdir(topworkdir)
    logname = os.path.join(topworkdir, 'docheck.log')
    logger = logging.getLogger('docheck')
    hdlr = logging.FileHandler(logname, 'w')
    logger.addHandler(hdlr)
    testbunch(directory, filename, topworkdir)
    logging.shutdown()
    if os.path.isdir(topworkdir) and os.path.isfile(logname) and \
               os.stat(logname).st_size == 0:
        shutil.rmtree(topworkdir)

if __name__ == '__main__':
    filestuff()
