#! /usr/bin/env python

import os, sys
import TokenTree
import strip_preprocs, unstrip_preprocs
from myexceptions import *
from LN import LN, MLLN

def Run(filename):
    """Does end-to-end processing of a C file.

    The five stages are: strip_preprocs, UnparsePlus, makenest, makesrc, and
    unstrip_preprocs.
    """
    stripfile = '%s.strip' % filename
    ppsfile = '%s.pps' % filename
    datafile = '%s.data' % filename
    errfile = '%s.err' % filename
    nestfile = '%s.nest' % filename
    tmpsrcfile = '%s.tmpsrc' % filename
    srcfile = '%s.src' % filename
    temperr = 'temp.err'
    if not os.path.isfile(filename):
        raise IOError, 'file not found: %s' % filename

    # Strip out the preprocessor directives.
    strip_preprocs.stripit(filename, stripfile, ppsfile)
    if os.path.exists(errfile):
        os.remove(errfile)

    # Run "UnparsePlus.java". This creates a ".data" file.
    if os.path.exists(temperr):
        os.remove(temperr)
    command = 'java UnparsePlus %s %s %s &> %s' % \
          (stripfile, datafile, errfile, temperr)
    os.system(command)
    if os.path.exists(temperr):
        f = open(temperr, 'r')
        text = f.read()
        f.close()
        os.remove(temperr)
        if len(text) > 0:
            raise JavaError, text
    if os.path.exists(errfile):
        if os.stat(errfile).st_size == 0:
            os.remove(errfile)
        else:
            f = open(errfile, 'r')
            text = f.read()
            f.close()
            os.remove(errfile)
            raise JavaException, text

    # Read in the ".data" file.
    text = open(datafile, 'r').read()

    # Convert it into a tree.
    tree = TokenTree.TokenTree(text)

    # Create the ".nest" file.
    textout = TokenTree.MakeNest(tree)
    open(nestfile, 'w').write(textout)

    # Reconstruct the source code and put it in a ".tmpsrc" file.
    textout = tree.make_source()  # I don't care if I mess up the tree.
    open(tmpsrcfile, 'w').write(textout)

    # Put the preprocessor directives back in.
    unstrip_preprocs.unstripit(ppsfile, tmpsrcfile, srcfile)

if __name__ == '__main__':
    # If "nest" or "src" is given, it is assumed that the ".data" file
    # already exists.
    if len(sys.argv) not in (2, 3):
        sys.exit('USAGE: ./makedata.py <filename> [<dowhat>] ' \
           'where <dowhat> can be "nest" or "source"')
    if len(sys.argv) == 3:
        filename = sys.argv[1]
        datafile = filename + '.data'
        nestfile = filename + '.nest'
        tmpsrcfile = filename + '.tmpsrc'
        text = open(datafile, 'r').read()
        tree = TokenTree(text)

        if sys.argv[2] == 'nest':
            textout = MakeNest(tree)
            open(nestfile, 'w').write(textout)
        elif sys.argv[2] == 'source':
            textout = tree.make_source()
            open(tmpsrcfile, 'w').write(textout)
        else:
            sys.exit('If present, the second argument must be ' \
                       '"nest" or "source"')
    else:
        Run(sys.argv[1])
