#! /usr/bin/env python

import sys

def _unindent(text, spaces=sys.maxint):
    """Unindents text as much as possible but by at most "n" spaces"""
    lines = text.splitlines()
    trivial = True
    for line in lines:
        if len(line.strip()) != 0:
            trivial = False
            n = len(line) - len(line.lstrip())
            spaces = min(n, spaces)
    out = []
    for line in lines:
        if trivial or len(line.strip()) == 0:
            out.append('')
        else:
            out.append(line[spaces:])
    return '\n'.join(out)

def _strip_const_semi(node):
    out = []
    for string in node.tree.tokenstrings:
        if string not in ('const', ';'):
            out.append(string)
    return out

# The templates are indented for readability. The program will unindent them.
templatedict = {
    ('basicStruct', None): """\
        <STUB: basicStruct>""",

    ('noNameYet', None): """\
        <STUB: noNameYet>""",

    ('simpleDeclaration', None): """\
        <STUB: simpleDeclaration>""",

    ('typedefEnum', None): """\
        <STUB: typedefEnum>""",

    ('typedefSimple', None): """\
        <STUB: typedefSimple>""",

    ('typedefStruct', None): """\
        <STUB: typedefStruct>""",

    ('typedefUnion', None): """\
        <STUB: typedefUnion>""",

    ('unknown', None): """\
        <STUB: unknown>""",
}

# Unindent the templates.
for pair in templatedict:
    templatedict[pair] = _unindent(templatedict[pair])

class Creators(object):
    def __init__(self, Q):
        self.Q = Q

    def top(self, node):
        if self.Q.IsPrototype():
            return '<@do_proto@>'
        if self.Q.IsFunctionPointer():
            return '<@do_fptr@>'
        if self.Q.IsTypedefStruct():
            return '<@typedefStruct@>'
        if self.Q.IsTypedefEnum():
            return '<@typedefEnum@>'
        if self.Q.IsTypedefUnion():
            return '<@typedefUnion@>'
        if self.Q.IsTypedefSimple():
            return '<@typedefSimple@>'
        if self.Q.IsSimpleDeclaration():
            return '<@simpleDeclaration@>'
        bs = self.Q.BasicStruct()
        if bs.failure:
            return '<@unknown@>'
        fd = self.Q.ForwardDeclaration(bs)
        if fd.success:
            return '<@do_forward@>'
        else:
            return '<@basicStruct@>'

    def do_proto(self, node):
        # Do not need the full parsing mechanism here.
        strings = _strip_const_semi(node)
        return '    ' + ' '.join(strings)

    def do_fptr(self, node):
        # Do not need the full parsing mechanism here.
        strings = _strip_const_semi(node)
        if strings[0] == 'typedef':
            strings[0] = 'ctypedef'
        return '    ' + ' '.join(strings)

    def do_forward(self, node):
        # Do not need the full parsing mechanism here.
        if len(node.tree.tokenstrings) != 3 or \
              node.tree.tokenstrings[0] not in ('struct', 'union'):
            raise RuntimeError, 'BUG in ugly code'
        return '    %s %s' % (node.tree.tokenstrings[0], node.tree.tokenstrings[1])

if __name__ == '__main__':
    import questioner, TokenTree
    Q = questioner.Questioner(TokenTree.TokenTree())
    C = Creators(Q)
    print dir(C)
    for key in templatedict:
        print key
        print templatedict[key]
        print
