#! /usr/bin/env python

# From the Python tutorial, 9.0:
# 
# "Although scopes are determined statically, they are used dynamically. At
# any time during execution, there are at least three nested scopes whose
# namespaces are directly accessible: the innermost scope, which is searched
# first, contains the local names; the namespaces of any enclosing
# functions, which are searched starting with the nearest enclosing scope;
# the middle scope, searched next, contains the current module's global
# names; and the outermost scope (searched last) is the namespace containing
# built-in names."

# This is a form of lexical scoping. When "wrapped" is called, it can see
# "wrap.cache". But "wrap.cache" is not in "wrapped.__dict__".

def wrap(f):
    wrap.cache = dict()

    def wrapped(num):
        print 'L21', num, wrapped.hits
        if num in wrap.cache:
            print 'cached'
            print 'L23', num, wrapped.hits
            wrapped.hits[num] += 1
            return wrap.cache[num]
        else:
            result = f(num)
            wrap.cache[num] = result
            wrapped.hits[num] = 1
            print 'L29', num, wrapped.hits
            return result

    wrapped.hits = dict()
    return wrapped

def fun(num):
    return num+1
wfun = wrap(fun)

def gun(num):
    return 100*num
wgun = wrap(gun)

print wfun(1)
print 'L46', wrap.cache
print 'L47', wfun.__dict__
print '------'
print 'L48', wgun.__dict__
print wgun(1)
print 'L50', wrap.cache

"""
print wfun(3)
print wgun(3)
print wfun(1)
print wgun(1)
print dir(wfun)
print wfun.__dict__
print wfun.hits
"""