#! /usr/bin/env python

from LN import LN, MLLN
import sys
import globals, TokenTree, questioner, TemplateTree, Creators

"""Notes:
An example of a standard prototype is:
    int fun1(int i);
The following is legal:
    int (fun2)(int i);
I am going to ignore this case in my code because it seldom occurs. In a
finished product, it should be included.
Here is a pointer to a function:
    int (*fun3)(int i);
See "GnuCEmitter.g.fixed" for "declarator".

A typical natural question to ask is:
    "Does this tree represent a prototype?
I find it unnatural to ask:
    "What do I have to do at a "declarator" node to answer all the
    questions that I might ask?

If the tree manipulations are done as actions in the grammar, they will
depend on global and tree-level messages. The actions may be 50 lines of
code.
"""
def strip_const_semi(tokenstrings):
    out = []
    for string in tokenstrings:
        if string != 'const':
            out.append(string)
    del out[-1]
    return out

def main():
    if len(sys.argv) != 2:
        sys.exit('USAGE: ./transform.py <C file>')
    base_filename = sys.argv[1]
    datafile = base_filename + '.data'
    text = open(datafile, 'r').read()

    # Parse the header file. Split it into pieces.
    bigtree = TokenTree.TokenTree(text)
    Q = questioner.Questioner(bigtree)
    globals.GlobalData.templatedict = Creators.templatedict
    globals.GlobalData.alltrees = Q.ExternalDef()

    for tree in globals.GlobalData.alltrees:
        tree.strip_special_tokens()
#        MLLN(tree)
        templatetree = TemplateTree.TemplateTree(tree, globals.GlobalData)
        templatetree.top.children_from_template()
        templatetree.top.text_assembler()
        MLLN(templatetree)
        print templatetree.top.text
        print '==========================================='
#    MLLN(globals.GlobalData.forward_declarations)

if __name__ == '__main__':
    main()
