#! /usr/bin/env python

import os, sys
import Node
from Node import CODE
from myexceptions import *
from LN import LN, MLLN

#>=== class Tree =================================================

class Tree(object):
    counter = 0

    def __init__(self, top=None):
        self.id = Tree.counter
        Tree.counter += 1
        if top is None:
            self.top = None
        elif isinstance(top, Node.Node):
            self.addnode(top, 'TOP')
            top.tree = self
        elif isinstance(top, Tree):
            self.copy(top)
        else:
            raise TypeError, '"top" must be a Node, a Tree, or None'

    # Create ".classname" attribute for this class and all its descendents.
    def get_classname(self):
        return self.__class__.__name__
    classname = property(get_classname)

    def __str__(self):
        if self.top is None:
            return 'Empty Tree'
        out = []
        for node in self.walk():
            lines = str(node).splitlines()
            for line in lines:
                s = '%s%s' % (node.depth * '  ', line)
                out.append(s)
        text = '\n'.join(out)
        return '<%s: %s>' % (self.classname, text)

    def __repr__(self):
        if self.top is None:
            return 'Empty Tree'
        out = []
        for node in self.walk():
            lines = repr(node).splitlines()
            for line in lines:
                s = '%s%s' % (node.depth * '  ', line)
                out.append(s)
        text = '\n'.join(out)
        return '<%s: %s>' % (self.classname, text)

    def copy(self, tree):
        """Copies a tree and all of its Nodes into self."""
        if tree.top is None:
            self.top = None
            return
        self.top = tree.top.copy()
        self.top.tree = self

        def recurse(oldnode, newnode):
            newnode.children = []
            for child in oldnode.children:
                newchild = child.copy()
                self.addnode(newchild, newnode, 'LAST')
                if not child.isleaf():
                    recurse(child, newchild)

        recurse(tree.top, self.top)
        for node in self.walk():
            node.top = self.top
            node.tree = self

    def show_walkaround(self):
        """Like "__str__" but with "walkaround" instead of "walk"."""
        out = []
        for code, node in self.walkaround():
            text = '%s%s' % (node.depth * '  ', str(node))
            out.append(text)
        return '\n'.join(out)

    def show_reverse_walk(self):
        """Like "__str__" but with "reverse_walk" instead of "walk"."""
        out = []
        for node in self.reverse_walk():
            text = '%s%s' % (node.depth * '  ', str(node))
            out.append(text)
        return '\n'.join(out)

    def walk(self, startnode=None):
        """Generator that does top-down, left-right traversal of tree."""
        if self.top is None:
            return
        if startnode is None:
            startnode = self.top
        for node in startnode.walk():
            yield node

    def reverse_walk(self, startnode=None):
        """Generator that does bottom-up, right-left traversal of tree.

        Use this if you plan to delete nodes.
        """
        if self.top is None:
            return
        if startnode is None:
            startnode = self.top
        for node in startnode.reverse_walk():
            yield node

    def walkaround(self, startnode=None):
        """Walks around the outside of a tree.

        Each Node node with code == CODE.NODENAME is visited twice. The
        first visit before its children are visited and corresponds to '('. 
        At the start of the first visit, self.walkaround_code is set to
        CODE.LP. The second visit is after its children are visited and
        corresponds to ')'. Just before the second visit,
        self.walkaround_code is set to WALKAROUND_CODE.SECOND. Some
        NODENAMEs may be leaves. They are still visited twice.
        """
        if self.top is None:
            return
        if startnode is None:
            startnode = self.top
        for code, node in startnode.walkaroundToken():
            yield code, node

    def walk_parents(self, startnode=None):
        """Walks a Tree, yielding the nodes and a list of their parents relative to startnode"""
        if self.top is None:
            return
        if startnode is None:
            startnode = self.top
        for pair in startnode.walk_parents():
            yield pair

    def addnode(self, node, parent, index='LAST'):
        """Add a Node to a Tree.

        "parent" can be a Node, or the string "TOP". "index" can be either
        an int or the string "LAST". When a node is added, it keeps its
        children.
        """
        if not isinstance(node, Node.Node):
            raise TypeError, '"node" must be a Node'
        if parent == 'TOP':
            self.top = node
            for node in self.top.walk():
                node.top = self.top
                node.tree = self
            self.top.set_depths(0)
            node.parent = None
            node.left = None
            node.right = None
            node.index = None
        elif not isinstance(parent, Node.Node):
            raise TypeError, '"parent" must be a Node or the string "TOP"'
        else:
            node.addnode(parent, index)

    def stripNODENAMEleaves(self):
        """Remove leaves that are also NODENAMEs. Modifies the tree.

        Use this function only after the tree is completed.
        """
        for node in self.reverse_walk():
            if node.isleaf() and node.code == CODE.NODENAME:
                node.kill()
