#! /usr/bin/env python

from __future__ import generators

DEBUG = False  # Displays recursive calls to "text_assembler". Also gives a
               # list of parents whenever a Node is processed.
COUNT = False  # Counts how often each "MF_" function is called."

import sys, time, treematch, match, Node, format_templates
from types import *
from linenum import LN

def is_typedef(top):
    """Is a declaration a typedef? """
    pattern = ("externalDef", "declaration", "declSpecifiers",
               "storageClassSpecifier")
    for obj in treematch.match(top, pattern):
        return obj[1]
    return None

def is_functionpointer(top):
    """Is a declaration a pointer to a function? """
    pattern = ("externalDef", "declaration", "initDeclList", "initDecl",
                "declarator", "declarator.1")
    obj = treematch.first_match(top, pattern)
    if obj is None:
        return False
    return True

def is_basicstruct(top):
    """Is a declaration a struct (as opposed to a typedef struct)"""
    pattern0 = ("externalDef", "declaration", "declSpecifiers", "typeSpecifier",
        "structSpecifier")
    pattern1 = ("externalDef", "declaration", "declSpecifiers", "typeSpecifier",
        "unionSpecifier")
    obj = treematch.first_match(top, pattern0)
    if obj is None:
        obj = treematch.first_match(top, pattern1)
    if obj is None:
        return False
    return True

#>--- typedefs, protos, and function pointers -----------------------

# Do not need the full parsing mechanism here. YET.
def do_proto(extDef):
    """Process prototypes"""
    tokens = list(treematch.gettokens(extDef))
    del tokens[-1]
    tokens = treematch.replace_spectokens(tokens)
    newtokens = []
    for token in tokens:
        if token not in ('nl1', 'nl2', 'nl3', 'nl4'):
            newtokens.append(token)
    print LN(), ' '.join(tokens)
    return '    ' + ' '.join(tokens)

# Do not need the full parsing mechanism here.
def do_fptr(extDef):
    """Process function pointers"""
    tokens = list(treematch.gettokens(extDef))
    del tokens[-1]
    tokens = treematch.replace_spectokens(tokens)
    newtokens = []
    for token in tokens:
        if token not in ('nl1', 'nl2', 'nl3', 'nl4'):
            newtokens.append(token)
    if tokens[0] == 'typedef':
        tokens[0] = 'ctypedef'
    else:
        print tokens
        raise Exception, 'bug'
    print LN(), ' '.join(tokens)
    return '    ' + ' '.join(tokens)

def do_structs(extDef, source, which, ts, all):
    """Extract data from a parse tree"""
    tree = Node.Tree('top', which, ts, all)
    tree.createNodes()
    text = tree.topnode.text_assembler()
    if which == 'decls':
        tree.all.finished_structs.append(ts.MF_Declarator_0())
        tree.all.forward_structs[ts.MF_Declarator_0()] = None
    # A Tree is full of cyclic references but it is referenced only in this
    # function. Give the garbage collector a hint.
    del tree
    return text

def do_typedefstruct(extDef, source, all):
    """Calls "do_struct" up to three times for each declaration"""
    out = []
    ts = match.Match(extDef, source)
    # These 3 are the nested stuff. They are done by hand in
    # "cvrest_nested.pyx".
    if ts.MF_Declarator_0() in ('CvMat_data', 'CvMatND_dim', 'CvEHMM_u'):
        return None
    # Make the Pyrex declaration.
    text = do_structs(extDef, source, 'decls', ts, all)
    out.append(text)

    tokens = treematch.gettokens(extDef)
    if tokens[0] == 'typedef' and \
             ts.MF_StructOrUnionBody_0() is None and \
             ts.MF_StructOrUnionBody_2() is None and \
             ts.MF_StructOrUnionBody_4() is None:
        # Make ToC and FromC functions for simple typedefs. Currently
        # needed only for "CvSubdiv2DEdge".
        if ts.MF_Declarator_0() == 'CvSubdiv2DEdge':
            text = do_structs(extDef, source, 'ToFromC', ts, all)
            out.append(text)
    else:
        # Make the class.
        if tokens[0] == 'typedef' and \
               not (ts.MF_StructOrUnionBody_0() is None and \
                    ts.MF_StructOrUnionBody_2() is None):
            text = do_structs(extDef, source, 'class', ts, all)
            out.append(text)
    if COUNT:
        print '-------'
        ts.print_counts()
        print
        ts.update_totals()
    return out

# Very ugly. Applies only to a few simple forward declarations. Do not need
# the full parsing mechanism here.
def do_basicstruct(extDef, source, all):
    """Ugly kluge"""
    tokens = treematch.gettokens(extDef)
    if len(tokens) != 3:
        raise Exception, 'BUG in ugly code'
    text = '    ' + ' '.join(tokens[:2])
    all.forward_structs[tokens[1]] = None
    return text

def classify(declaration):
    """Classifies a declaration"""
    if is_functionpointer(declaration):
        return 'fptr'
    elif is_typedef(declaration):
        return 'typedefstruct'
    elif is_basicstruct(declaration):
        return 'basicstruct'
    else:
        return 'proto'

def process_decl(code, extDef, source, all):
    """A switch on the result of classify"""
    if code == 'typedef':
        return do_typedef(extDef)
    elif code == 'proto':
        return do_proto(extDef)
    elif code == 'fptr':
        return do_fptr(extDef)
    elif code is 'typedefstruct':
        return do_typedefstruct(extDef, source, all)
    elif code is 'basicstruct':
        return do_basicstruct(extDef, source, all)
    else:
        raise ValueError, 'illegal code'

def doit():
    """main"""
    if len(sys.argv) != 2:
        print >> sys.stderr, 'program takes exactly one agrument'
        sys.exit(-1)
    root = sys.argv[1]
    header_file = root + '.h'
    include_file = 'tools.pyx'
    if header_file == 'cvrest.h':
        header_file = 'OpenCV-0.9.5.includes/cv.h'
        include_file = 'cvrest_nested.pyx'
    text = file(root + '.clean', 'r').read()
    top = eval(text)
    t = time.clock()
    templates_file_data = file('CREATORs', 'r').read()
    template_data, labels = format_templates.template_dict(templates_file_data)
    format_time = time.clock() - t
    t = time.clock()
    metanodes = dict()
    for label in labels:
        metanodes[label] = Node.MetaNode(label, template_data)
    MN_time = time.clock() - t

    header_template = """\
include "%s"

cdef extern from "%s":

    cdef enum TemP__srdgh:
        CV_MAX_DIM = 40"""

    out = [header_template % (include_file, header_file)]
    t = time.clock()
    pyrex_decls = []
    classes = []
    all = Node.All(template_data, labels, metanodes)
    for extDef, source in top:
        tokens = treematch.gettokens(extDef)
        # These two done elsewhere.
        if len(tokens) > 1 and tokens[-2] in ('size_t', 'uchar'):
            continue
#        print '=================================================='
        code = classify(extDef)
        obj = process_decl(code, extDef, source, all)
        if obj is None:
            continue
        if code == 'typedefstruct':
             pyrex_decls.append(obj[0])
             if len(obj) == 2:
                 classes.append(obj[1])
        else:
             pyrex_decls.append(obj)
        # If you want to explicitly collect garbage, do it here.
    out.extend(pyrex_decls)
    out.extend(classes)
    out.append('')
    bigtext = '\n\n'.join(out)
    print 'Format:', format_time
    print 'MetaNodes:', MN_time
    print time.clock() - t, 'CPU seconds.'
    if COUNT:
        match.Match.print_bigcounts()
    file(root + '.pyrex', 'w').write(bigtext)

if __name__ == '__main__':
    doit()
