from __future__ import generators

import sys, treematch
from treematch import OR, ANY, NoMatchException
from linenum import LN
from externalDef import COUNT

class UNUSED:
    pass

class Match(object):
    attribute_names = ('TypeSpecifier', 'StructUnionOrEnumSpecifier',
          'StructUnionOrEnumSpecifier_token', 'StructOrUnionBody',
          'StructOrUnionBody_0', 'StructOrUnionBody_2',
          'StructOrUnionBody_4', 'TypedefStructLabel',
          'StructDeclarationList', 'StructDeclaration_list', 'Declarator_0',
          'SimpleTypedefPointerGroup', 'StructDeclaration_info', 'EnumList')

    if COUNT:
        Totals = dict()
        for name in attribute_names:
            Totals[name] = 0

    def __init__(self, extDef, source):
        self.extDef = extDef
        self.simple_typedefs = {}
        self.simple_typedefs['normals'] = ('size_t', 'int64', 'uint64',
                'uchar', 'CvHistType', 'CVStatus', 'CvSubdiv2DEdge', 'CvArr')
        # "CvPoint2DSeq" is never used.
        self.simple_typedefs['structs'] = ('CvPoint2DSeq',)
        self.simple_typedefs['pointers'] = \
            {'CvVect32f' : 'float',
             'CvMatr32f' : 'float',
             'CvVect64d' : 'double',
             'CvMatr64d' : 'double'}

        self.original = treematch.prettyprint_source(source)

        if COUNT:
            self.counters = dict()
            for name in self.attribute_names:
                self.counters[name] = 0

        self.cache = dict()
        for name in self.attribute_names:
            self.cache[name] = UNUSED

    def update_totals(self):
        for name in self.attribute_names:
            self.Totals[name] += self.counters[name]

    def nothrow(self, name):
        try:
            t = getattr(self, name)
            return t
        except NoMatchException:
            return NOMATCH

    def nothrow2(self, name):
        try:
            return getattr(self, name)
        except NoMatchException:
            return 'No such attribute'

    def print_counts(self):
        for name in self.attribute_names:
            print name, self.counters[name]

    def print_bigcounts(self):
        for name in self.attribute_names:
            print name, self.Totals[name]
    print_bigcounts = classmethod(print_bigcounts)

    def part2string(self, names):
        out = []
        for name in names:
            if name != 'original':
                text = '%s: %s' % (name, self.nothrow2(name))
            else:
                text = 'original: """%s"""' % self.original
            out.append(text)
        out.append('')
        return '\n'.join(out)

    def MF_TypeSpecifier(self):
        if self.cache['TypeSpecifier'] is not UNUSED:
            return self.cache['TypeSpecifier']
        if COUNT:
            self.counters['TypeSpecifier'] += 1
        pattern = ("externalDef", "declaration", "declSpecifiers",
               "typeSpecifier")
        text = treematch.first_match(self.extDef, pattern)
        if text is None:
            raise NoMatchException, 'no match found'
        self.cache['TypeSpecifier'] = text
        return text

    def MF_Declarator_0(self):
        if self.cache['Declarator_0'] is not UNUSED:
            return self.cache['Declarator_0']
        if COUNT:
            self.counters['Declarator_0'] += 1
        pattern = ("externalDef", "declaration", "initDeclList",
           "initDecl", "declarator", "declarator.0")
        t = treematch.first_match(self.extDef, pattern)
        if t is None:
            raise NoMatchException, 'match failure one'
        text = treematch.first(t)
        if text == '':
            raise NoMatchException, 'match failure two'
        self.cache['Declarator_0'] = text
        return text

    def MF_SimpleTypedefPointerGroup(self):
        if self.cache['SimpleTypedefPointerGroup'] is not UNUSED:
            return self.cache['SimpleTypedefPointerGroup']
        if COUNT:
            self.counters['SimpleTypedefPointerGroup'] += 1
        pattern = ("externalDef", "declaration", "initDeclList",
           "initDecl", "declarator", "pointerGroup")
        pg = treematch.first_match(self.extDef, pattern)
        if pg is None:
            t = None
        else:
            t = treematch.first(pg)
        self.cache['SimpleTypedefPointerGroup'] = t
        return t

    #>--- enum ----------------------------------------------------

    def MF_EnumList(self):
        if self.cache['EnumList'] is not UNUSED:
            return self.cache['EnumList']
        if COUNT:
            self.counters['EnumList'] += 1
        pattern = ("externalDef", "declaration", "declSpecifiers",
               "typeSpecifier", "enumSpecifier", "enumList")
        el = treematch.first_match(self.extDef, pattern)
        if el is None:
            raise NoMatchException, 'match failed'
        self.cache['EnumList'] = el
        return el

    #>--- struct and union-----------------------------------------

    def MF_StructUnionOrEnumSpecifier(self):
        if self.cache['StructUnionOrEnumSpecifier'] is not UNUSED:
            return self.cache['StructUnionOrEnumSpecifier']
        if COUNT:
            self.counters['StructUnionOrEnumSpecifier'] += 1
        ORpatt = (OR, "structSpecifier", "unionSpecifier", "enumSpecifier")
        pattern = ("typeSpecifier", ORpatt)
        typeSpec = self.MF_TypeSpecifier()
        sues = treematch.first_match(typeSpec, pattern)
        if sues is None:
            raise NoMatchException, 'match failed'
        self.cache['StructUnionOrEnumSpecifier'] = sues
        return sues

    def MF_StructUnionOrEnumSpecifier_token(self):
        if self.cache['StructUnionOrEnumSpecifier_token'] is not UNUSED:
            return self.cache['StructUnionOrEnumSpecifier_token']
        if COUNT:
            self.counters['StructUnionOrEnumSpecifier_token'] += 1
        sues = self.MF_StructUnionOrEnumSpecifier()
        if isinstance(sues, (list, tuple)) and len(sues) > 1:
            t = sues[1]
        else:
            raise NoMatchException, 'match failed'
        self.cache['StructUnionOrEnumSpecifier_token'] = t
        return t

    def MF_StructOrUnionBody(self):
        if self.cache['StructOrUnionBody'] is not UNUSED:
            return self.cache['StructOrUnionBody']
        if COUNT:
            self.counters['StructOrUnionBody'] += 1
        pattern = (ANY, 'structOrUnionBody')
        sues = self.MF_StructUnionOrEnumSpecifier()
        soub = treematch.first_match(sues, pattern)
        if soub is None:
            raise NoMatchException, 'match failed'
        self.cache['StructOrUnionBody'] = soub
        return soub

    def MF_StructOrUnionBody_0(self):
        if self.cache['StructOrUnionBody_0'] is not UNUSED:
            return self.cache['StructOrUnionBody_0']
        if COUNT:
            self.counters['StructOrUnionBody_0'] += 1
        pattern = ("structOrUnionBody", "structOrUnionBody.0")
        try:
            soub = self.MF_StructOrUnionBody()
        except NoMatchException:
            self.cache['StructOrUnionBody_0'] = None
            return None
        fm = treematch.first_match(soub, pattern)
        if fm is None:
            self.cache['StructOrUnionBody_0'] = None
            return None
        t = treematch.first(fm)
        self.cache['StructOrUnionBody_0'] = t
        return t

    def MF_StructOrUnionBody_2(self):
        if self.cache['StructOrUnionBody_2'] is not UNUSED:
            return self.cache['StructOrUnionBody_2']
        if COUNT:
            self.counters['StructOrUnionBody_2'] += 1
        pattern = ("structOrUnionBody", "structOrUnionBody.2")
        try:
            soub = self.MF_StructOrUnionBody()
        except NoMatchException:
            self.cache['StructOrUnionBody_2'] = None
            return None
        fm = treematch.first_match(soub, pattern)
        if fm is None:
            self.cache['StructOrUnionBody_2'] = None
            return None
        t = treematch.first(fm)
        self.cache['StructOrUnionBody_2'] = t
        return t

    def MF_StructOrUnionBody_4(self):
        if self.cache['StructOrUnionBody_4'] is not UNUSED:
            return self.cache['StructOrUnionBody_4']
        if COUNT:
            self.counters['StructOrUnionBody_4'] += 1
        pattern = ("structOrUnionBody", "structOrUnionBody.4")
        try: 
            soub = self.MF_StructOrUnionBody()
        except NoMatchException:
            self.cache['StructOrUnionBody_4'] = None
            return None
        fm = treematch.first_match(soub, pattern)
        if fm is None:
            self.cache['StructOrUnionBody_4'] = None
            return None
        return treematch.first(fm)

    def MF_TypedefStructLabel(self):
        if self.cache['TypedefStructLabel'] is not UNUSED:
            return self.cache['TypedefStructLabel']
        if COUNT:
            self.counters['TypedefStructLabel'] += 1
        d0 = self.MF_Declarator_0()
        if d0 == '':
            t = self.MF_StructOrUnionBody_4()
        else:
            t = d0
        self.cache['TypedefStructLabel'] = t
        return t

    def MF_StructDeclarationList(self):
        if self.cache['StructDeclarationList'] is not UNUSED:
            return self.cache['StructDeclarationList']
        if COUNT:
            self.counters['StructDeclarationList'] += 1
        sues = self.MF_StructUnionOrEnumSpecifier()
        pattern = (ANY, "structOrUnionBody", "structDeclarationList")
        obj = treematch.first_match(sues, pattern)
        if obj is None:
            raise NoMatchException, 'match failed'
        self.cache['StructDeclarationList'] = obj
        return obj

    def MF_StructDeclaration_list(self):
        if self.cache['StructDeclaration_list'] is not UNUSED:
            return self.cache['StructDeclaration_list']
        if COUNT:
            self.counters['StructDeclaration_list'] += 1
        try:
            sdl = self.MF_StructDeclarationList()
        except NoMatchException:
            self.cache['StructDeclaration_list'] = []
            return []
        pattern = ("structDeclarationList", "structDeclaration")
        out = treematch.all_matches(sdl, pattern)
        self.cache['StructDeclaration_list'] = out
        return out

    def StructDeclaration_info_single(self, top):
        if top[0] != "structDeclaration":
            raise ValueError, 'Not a "structDeclaration". It is a "%s".' % top[0]
        sql = treematch.first_match(top,
                 ("structDeclaration", "specifierQualifierList"))
        if sql is None:
            return []
#            raise NoMatchException, 'match failure one'
        spec = treematch.gettokens(sql)
        tnpatt = ("structDeclaration", "specifierQualifierList",
                  "typeSpecifier", "typedefName")
        tn = treematch.first_match(top, tnpatt)
        if tn is None or tn in self.simple_typedefs['normals'] or \
                tn in self.simple_typedefs['pointers']:
            isTypedefstruct_name = False
        else:
            isTypedefstruct_name = True
        out = []
        sdl = treematch.first_match(top,
                 ("structDeclaration", "structDeclaratorList"))
        if sdl is None:
            return []
#            raise NoMatchException, 'match failure two'
        for decl in treematch.match(sdl,
                 ("structDeclaratorList", "structDeclarator", "declarator")):
            starsobj = treematch.first_match(decl, ("declarator", "pointerGroup"))
            if starsobj is None:
                stars = ''
                start = 2
            else:
                stars = treematch.gettokens_join(starsobj)
                start = 3
            parts = []
            dims = []
            for obj in decl[start:]:
                dim = treematch.gettokens_join(obj)
                parts.append(dim)
                if dim[0] not in ('[', ']'):
                    dims.append(dim)
            dims_str = ' '.join(parts)
            varobj = treematch.first_match(decl, ("declarator", "declarator.0"))
            if varobj is None:
                raise NoMatchException, 'match failure three'
            var =  treematch.gettokens_join(varobj)
            tup = (isTypedefstruct_name, spec, stars, var, dims_str, dims)
            out.append(tup)
        return out

    def MF_StructDeclaration_info(self):
        if self.cache['StructDeclaration_info'] is not UNUSED:
            return self.cache['StructDeclaration_info']
        if COUNT:
            self.counters['StructDeclaration_info'] += 1
        out = []
        for sd in self.MF_StructDeclaration_list():
            info = self.StructDeclaration_info_single(sd)
            out.append(info)
        tout = tuple(out)
        self.cache['StructDeclaration_info'] = tout
        return tout
