
"""
Most of the matching functions (those with the names "MF_...") return a
result that never changes. These functions need be called only once. But
sometimes the match will fail and NoMatchException will be thrown. So I
cache the throwing of NoMatchException by saving NOMATCH. In other words,
the "N_..." attributes remember if their creation threw a NoMatchException,
and rethrow it if accessed.

Here is a simple example of "makeprop":

    def makeprop(truename, funname):
        def fget(self):
            setattr(self, truename, getattr(self, funname)())
            return getattr(self, truename)
        return property(fget, None, None)

    class X(object):
        i = makeprop('_i', 'Fi')

        def __init__(self, k):
            self.k = k

        def Fi(self):
            return self.k

    x = X(11)
    print x.i
"""
def makeprop(truename, funname):
    def fget(self):
        try:
            value = getattr(self, truename)
            # If funname was called previously and threw NoMatchException,
            # the attribute "truename" was set to NOMATCH.
            if value == NOMATCH:
                raise NoMatchException, 'some match failed in %s' % funname
        except AttributeError:
            try:
                value =  getattr(self, funname)()
            except NoMatchException:
                setattr(self, truename, NOMATCH)
                raise
            setattr(self, truename, value)
        return value

    return property(fget, None, None)

def makepropEZ(name):
    return makeprop('_' + name, 'MF_' + name)
